import os
import sys

from waflib import Utils, Options, Errors
from waflib.Configure import conf
from waflib.TaskGen import before_method, after_method, feature
from waflib.Logs import debug, warn

DISTUTILS_IMP = ['from distutils.sysconfig import get_config_var, get_python_lib']

@feature('pyext')
@before_method('apply_link', 'apply_bundle')
def set_bundle(self):
	if Utils.unversioned_sys_platform() == 'darwin':
		self.mac_bundle = True

@conf
def check_python_version(conf, minver=None):
	"""
	Check if the python interpreter is found matching a given minimum version.
	minver should be a tuple, eg. to check for python >= 2.4.2 pass (2,4,2) as minver.

	If successful, PYTHON_VERSION is defined as 'MAJOR.MINOR'
	(eg. '2.4') of the actual python version found, and PYTHONDIR is
	defined, pointing to the site-packages directory appropriate for
	this python version, where modules/packages/extensions should be
	installed.

	:param minver: minimum version
	:type minver: tuple of int
	"""
	assert minver is None or isinstance(minver, tuple)
	pybin = conf.env['PYTHON']
	if not pybin:
		conf.fatal('could not find the python executable')

	# Get python version string
	cmd = pybin + ['-c', 'import sys\nfor x in sys.version_info: print(str(x))']
	debug('python: Running python command %r' % cmd)
	lines = conf.cmd_and_log(cmd).split()
	assert len(lines) == 5, "found %i lines, expected 5: %r" % (len(lines), lines)
	pyver_tuple = (int(lines[0]), int(lines[1]), int(lines[2]), lines[3], int(lines[4]))

	# compare python version with the minimum required
	result = (minver is None) or (pyver_tuple >= minver)

	if result:
		# define useful environment variables
		pyver = '.'.join([str(x) for x in pyver_tuple[:2]])
		conf.env['PYTHON_VERSION'] = pyver

	# Feedback
	pyver_full = '.'.join(map(str, pyver_tuple[:3]))
	if minver is None:
		conf.msg('Checking for python version', pyver_full)
	else:
		minver_str = '.'.join(map(str, minver))
		conf.msg('Checking for python version', pyver_tuple, ">= %s" % (minver_str,) and 'GREEN' or 'YELLOW')

	if not result:
		conf.fatal('The python version is too old, expecting %r' % (minver,))

@conf
def get_python_variables(self, variables, imports=None):
	"""
	Spawn a new python process to dump configuration variables

	:param variables: variables to print
	:type variables: list of string
	:param imports: one import by element
	:type imports: list of string
	:return: the variable values
	:rtype: list of string
	"""
	if not imports:
		try:
			imports = self.python_imports
		except AttributeError:
			imports = DISTUTILS_IMP

	program = list(imports) # copy
	program.append('')
	for v in variables:
		program.append("print(repr(%s))" % v)
	os_env = dict(os.environ)
	try:
		del os_env['MACOSX_DEPLOYMENT_TARGET'] # see comments in the OSX tool
	except KeyError:
		pass

	try:
		out = self.cmd_and_log(self.env.PYTHON + ['-c', '\n'.join(program)], env=os_env)
	except Errors.WafError:
		self.fatal('The distutils module is unusable: install "python-devel"?')
	return_values = []
	for s in out.split('\n'):
		s = s.strip()
		if not s:
			continue
		if s == 'None':
			return_values.append(None)
		elif s[0] == "'" and s[-1] == "'":
			return_values.append(s[1:-1])
		elif s[0].isdigit():
			return_values.append(int(s))
		else: break
	return return_values

@conf
def check_python_headers(conf):
	"""
	Check for headers and libraries necessary to extend or embed python by using the module *distutils*.
	On success the environment variables xxx_PYEXT and xxx_PYEMBED are added:

	* PYEXT: for compiling python extensions
	* PYEMBED: for embedding a python interpreter
	"""

	# FIXME rewrite

	if not conf.env['CC_NAME'] and not conf.env['CXX_NAME']:
		conf.fatal('load a compiler first (gcc, g++, ..)')

	if not conf.env['PYTHON_VERSION']:
		conf.check_python_version()

	env = conf.env
	pybin = env.PYTHON
	if not pybin:
		conf.fatal('could not find the python executable')

	v = 'INCLUDEPY SO LDFLAGS MACOSX_DEPLOYMENT_TARGET LDSHARED CFLAGS'.split()
	try:
		lst = conf.get_python_variables(["get_config_var('%s') or ''" % x for x in v])
	except RuntimeError:
		conf.fatal("Python development headers not found (-v for details).")

	vals = ['%s = %r' % (x, y) for (x, y) in zip(v, lst)]
	conf.to_log("Configuration returned from %r:\n%r\n" % (pybin, '\n'.join(vals)))

	dct = dict(zip(v, lst))
	x = 'MACOSX_DEPLOYMENT_TARGET'
	if dct[x]:
		conf.env[x] = conf.environ[x] = dct[x]

	env['pyext_PATTERN'] = '%s' + dct['SO'] # not a mistake

	if Options.options.use_distutils_flags:
		all_flags = dct['LDFLAGS'] + ' ' + dct['LDSHARED'] + ' ' + dct['CFLAGS']
		conf.parse_flags(all_flags, 'PYEXT')

	env['INCLUDES_PYEXT'] = [dct['INCLUDEPY']]

@feature('pyext')
@before_method('propagate_uselib_vars', 'apply_link')
@after_method('apply_bundle')
def init_pyext(self):
	"""
	Change the values of *cshlib_PATTERN* and *cxxshlib_PATTERN* to remove the
	*lib* prefix from library names.
	"""
	self.uselib = self.to_list(getattr(self, 'uselib', []))
	if not 'PYEXT' in self.uselib:
		self.uselib.append('PYEXT')
	# override shlib_PATTERN set by the osx module
	self.env['cshlib_PATTERN'] = self.env['cxxshlib_PATTERN'] = self.env['macbundle_PATTERN'] = self.env['pyext_PATTERN']

	try:
		if not self.install_path:
			return
	except AttributeError:
		self.install_path = '${PYTHONARCHDIR}'

def configure(conf):
	"""
	Detect the python interpreter
	"""
	default = [sys.executable]
	try:
		conf.find_program('python', var='PYTHON')
	except conf.errors.ConfigurationError:
		warn("could not find a python executable, setting to sys.executable '%s'" % sys.executable)
		conf.env.PYTHON = default

	if conf.env.PYTHON != default:
		warn("python executable '%s' different from sys.executable '%s'" % (conf.env.PYTHON, default))
	conf.env.PYTHON = conf.cmd_to_list(default)

	v = conf.env
	v['PYCMD'] = '"import sys, py_compile;py_compile.compile(sys.argv[1], sys.argv[2])"'
	v['PYFLAGS'] = ''
	v['PYFLAGS_OPT'] = '-O'

def options(opt):
	opt.add_option('--use-distutils-flags',
			action='store_true',
			default=False,
			help='Use flags as defined by distutils [Default:no]',
			dest='pyc')



import os
import tempfile

import os.path as op

import mock

from bento.core.node \
    import \
        create_base_nodes
from bento.core.pkg_objects \
    import \
        Executable
from bento.compat.api.moves \
    import \
        unittest
from bento.commands.script_utils \
    import \
        nt_quote_arg, create_win32_script, create_posix_script, create_scripts

class TestMisc(unittest.TestCase):
    def test_nt_quote_arg(self):
        s = nt_quote_arg("foo")
        self.assertEqual(s, "foo")

        s = nt_quote_arg(" foo")
        self.assertEqual(s, '" foo"')

        s = nt_quote_arg("\ foo")
        self.assertEqual(s, '"\\ foo"')

class TestCreateScript(unittest.TestCase):
    def setUp(self):
        super(TestCreateScript, self).setUp()

        self.old_cwd = os.getcwd()
        self.d = tempfile.mkdtemp()

        try:
            os.chdir(self.d)
            self.top_node, self.build_node, self.run_node = create_base_nodes(self.d, op.join(self.d, "build"))
            self.build_node.mkdir()
        except:
            os.chdir(self.old_cwd)

    def tearDown(self):
        os.chdir(self.old_cwd)
        super(TestCreateScript, self).tearDown()

    def _test_create_script(self, func, executable, r_list):
        nodes = func("foo", executable, self.build_node)
        self.assertEqual(set(self.build_node.listdir()), set(r_list))
        self.assertEqual(set([node.path_from(self.build_node) for node in nodes]), set(r_list))

    def _test_create_scripts(self, executable, r_list):
        res = create_scripts({"foo": executable}, self.build_node)
        nodes = res["foo"]
        self.assertEqual(set(self.build_node.listdir()), set(r_list))
        self.assertEqual(set([node.path_from(self.build_node) for node in nodes]), set(r_list))

    @mock.patch("sys.platform", "win32")
    def test_create_script_on_win32(self):
        r_list = set(["foo-script.py", "foo.exe", "foo.exe.manifest"])
        executable = Executable("foo", "foo.bar", "fubar")
        self._test_create_scripts(executable, r_list)

    @mock.patch("sys.platform", "darwin")
    def test_create_script_on_posix(self):
        r_list = set(["foo"])
        executable = Executable("foo", "foo.bar", "fubar")
        self._test_create_scripts(executable, r_list)

    def test_create_posix_script(self):
        r_list = ["foo"]
        executable = Executable("foo", "foo.bar", "fubar")
        self._test_create_script(create_posix_script, executable, r_list)

    def test_create_win32_script(self):
        executable = Executable("foo", "foo.bar", "fubar")
        r_list = set(["foo-script.py", "foo.exe", "foo.exe.manifest"])
        self._test_create_script(create_win32_script, executable, r_list)



#! /usr/bin/env python
#import demandimport
#demandimport.enable()
import sys
import os
import traceback
import warnings

import bento

from bento.utils.utils \
    import \
        pprint, extract_exception
from bento._config \
    import \
        BENTO_SCRIPT, DB_FILE, _SUB_BUILD_DIR
from bento.core \
    import \
        PackageDescription
from bento.compat.api \
    import \
        defaultdict
import bento.core.node

from bento.commands.build \
    import \
        BuildCommand
from bento.commands.build_egg \
    import \
        BuildEggCommand
from bento.commands.build_pkg_info \
    import \
        BuildPkgInfoCommand
from bento.commands.build_wininst \
    import \
        BuildWininstCommand
from bento.commands.configure \
    import \
        ConfigureCommand
from bento.commands.core \
    import \
        HelpCommand
from bento.commands.dependency \
    import \
        CommandScheduler
from bento.commands.hooks \
    import \
        find_pre_hooks, find_post_hooks, find_startup_hooks, \
        find_shutdown_hooks, find_options_hooks, find_command_hooks
from bento.commands.install \
    import \
        InstallCommand
from bento.commands.parse \
    import \
        ParseCommand
from bento.commands.register \
    import \
        RegisterPyPI
from bento.commands.registries \
    import \
        CommandRegistry, ContextRegistry, OptionsRegistry
from bento.commands.sdist \
    import \
        SdistCommand
from bento.commands.sphinx_command \
    import \
        SphinxCommand
from bento.commands.upload \
    import \
        UploadPyPI
from bento.commands.options \
    import \
        OptionsContext, Option

from bento.backends.utils \
    import \
        load_backend
from bento.backends.yaku_backend \
    import \
        BuildYakuContext, ConfigureYakuContext
from bento.commands.command_contexts \
    import \
        HelpContext, SdistContext, ContextWithBuildDirectory
from bento.commands.wrapper_utils \
    import \
        set_main, run_with_dependencies
from bento.commands.contexts \
    import \
        GlobalContext
from bento.convert \
    import \
        ConvertCommand, DetectTypeCommand
import bento.errors

from bentomakerlib.package_cache \
    import \
        CachedPackage
from bentomakerlib.help \
    import \
        get_usage

if os.environ.get("BENTOMAKER_DEBUG", "0") != "0":
    BENTOMAKER_DEBUG = True
else:
    BENTOMAKER_DEBUG = False

SCRIPT_NAME = 'bentomaker'

# Path relative to build directory
CMD_DATA_DUMP = os.path.join(_SUB_BUILD_DIR, "cmd_data.db")

class GlobalOptions(object):
    def __init__(self, cmd_name, cmd_argv, show_usage, build_directory,
            bento_info, show_version, show_full_version, disable_autoconfigure):
        self.cmd_name = cmd_name
        self.cmd_argv = cmd_argv
        self.show_usage = show_usage
        self.build_directory = build_directory
        self.bento_info = bento_info
        self.show_version = show_version
        self.show_full_version = show_full_version
        self.disable_autoconfigure = disable_autoconfigure

#================================
#   Create the command line UI
#================================
def register_commands(global_context):
    global_context.register_command("help", HelpCommand())
    global_context.register_command("configure", ConfigureCommand())
    global_context.register_command("build", BuildCommand())
    global_context.register_command("install", InstallCommand())
    global_context.register_command("convert", ConvertCommand())
    global_context.register_command("sdist", SdistCommand())
    global_context.register_command("build_egg", BuildEggCommand())
    global_context.register_command("build_wininst", BuildWininstCommand())
    global_context.register_command("sphinx", SphinxCommand())
    global_context.register_command("register_pypi", RegisterPyPI())
    global_context.register_command("upload_pypi", UploadPyPI())

    global_context.register_command("build_pkg_info", BuildPkgInfoCommand(), public=False)
    global_context.register_command("parse", ParseCommand(), public=False)
    global_context.register_command("detect_type", DetectTypeCommand(), public=False)
 
    if sys.platform == "darwin":
        import bento.commands.build_mpkg
        global_context.register_command("build_mpkg",
            bento.commands.build_mpkg.BuildMpkgCommand(), public=False)
        global_context.set_before("build_mpkg", "build")

def register_options(global_context, cmd_name):
    """Register options for the given command."""
    cmd = global_context.retrieve_command(cmd_name)
    context = OptionsContext.from_command(cmd)

    if not global_context.is_options_context_registered(cmd_name):
        global_context.register_options_context(cmd_name, context)

def register_options_special(global_context):
    # Register options for special topics not attached to a "real" command
    # (e.g. 'commands')
    context = OptionsContext()
    def print_usage():
        print(get_usage(global_context))
    context.parser.print_help = print_usage
    global_context.register_options_context_without_command("commands", context)

    context = OptionsContext()
    def print_help():
        global_options = global_context.retrieve_options_context("")
        p = global_options.parser
        return(p.print_help())
    context.parser.print_help = print_help
    global_context.register_options_context_without_command("globals", context)

def register_command_contexts(global_context):
   # global_context.register_default_context(CmdContext)
    default_mapping = defaultdict(lambda: ContextWithBuildDirectory)
    default_mapping.update(dict([
            ("configure", ConfigureYakuContext),
            ("build", BuildYakuContext),
            ("build_egg", ContextWithBuildDirectory),
            ("build_wininst", ContextWithBuildDirectory),
            ("build_mpkg", ContextWithBuildDirectory),
            ("install", ContextWithBuildDirectory),
            ("sdist", SdistContext),
            ("help", HelpContext)]))

    for cmd_name in global_context.command_names(public_only=False):
        if not global_context.is_command_context_registered(cmd_name):
            global_context.register_command_context(cmd_name, default_mapping[cmd_name])

# All the global state/registration stuff goes here
def register_stuff(global_context):
    register_commands(global_context)
    register_options_special(global_context)
    register_command_contexts(global_context)

def main(argv=None):
    if hasattr(os, "getuid"):
        if os.getuid() == 0:
            pprint("RED", "Using bentomaker under root/sudo is *strongly* discouraged - do you want to continue ? y/N")
            ans = raw_input()
            if not ans.lower() in ["y", "yes"]:
                raise bento.errors.UsageException("bentomaker execution canceld (not using bentomaker with admin privileges)")

    if argv is None:
        argv = sys.argv[1:]

    options_context = create_global_options_context()
    popts = parse_global_options(options_context, argv)

    cmd_name = popts.cmd_name

    if popts.show_version:
        print(bento.__version__)
        return

    if popts.show_full_version:
        print(bento.__version__ + "git" + bento.__git_revision__)
        return

    # FIXME: top_node vs srcnode
    source_root = os.path.join(os.getcwd(), os.path.dirname(popts.bento_info))
    build_root = os.path.join(os.getcwd(), popts.build_directory)

    # FIXME: create_root_with_source_tree should return source node and build
    # node so that we don't have to find them and take the risk of
    # inconsistency
    root = bento.core.node.create_root_with_source_tree(source_root, build_root)
    run_node = root.find_node(os.getcwd())
    top_node = root.find_node(source_root)
    build_node = root.find_node(build_root)
    if run_node != top_node and run_node.is_src():
        raise bento.errors.UsageException("You cannot execute bentomaker in a subdirectory of the source tree !")
    if run_node != build_node and run_node.is_bld():
        raise bento.errors.UsageException("You cannot execute bentomaker in a subdirectory of the build tree !")

    global_context = GlobalContext(build_node.make_node(CMD_DATA_DUMP),
                                   CommandRegistry(), ContextRegistry(),
                                   OptionsRegistry(), CommandScheduler())
    global_context.register_options_context_without_command("", options_context)

    if not popts.disable_autoconfigure:
        global_context.set_before("build", "configure")
    global_context.set_before("build_egg", "build")
    global_context.set_before("build_wininst", "build")
    global_context.set_before("install", "build")

    if cmd_name and cmd_name not in ["convert"]:
        return _wrapped_main(global_context, popts, run_node, top_node, build_node)
    else:
        # XXX: is cached package necessary here ?
        cached_package = None
        register_stuff(global_context)
        for cmd_name in global_context.command_names():
            register_options(global_context, cmd_name)
        return _main(global_context, cached_package, popts, run_node, top_node, build_node)

def _wrapped_main(global_context, popts, run_node, top_node, build_node):
    # Some commands work without a bento description file (convert, help)
    # FIXME: this should not be called here then - clearly separate commands
    # which require bento.info from the ones who do not
    bento_info_node = top_node.find_node(BENTO_SCRIPT)
    if bento_info_node is not None:
        db_node = build_node.make_node(DB_FILE)
        cached_package = CachedPackage(db_node)
        package = cached_package.get_package(bento_info_node)
        package_options = cached_package.get_options(bento_info_node)

        if package.use_backends:
            if len(package.use_backends) > 1:
                raise ValueError("Only up to one backend supported for now")
            else:
                assert global_context.backend is None
                global_context.backend = load_backend(package.use_backends[0])()
        global_context.register_package_options(package_options)

        mods = set_main(package, top_node, build_node)

    else:
        warnings.warn("No %r file in current directory - only generic options "
                      "will be displayed" % BENTO_SCRIPT)
        cached_package = None
        package_options = None
        mods = []

    startup_hooks = find_startup_hooks(mods)
    option_hooks = find_options_hooks(mods)
    shutdown_hooks = find_shutdown_hooks(mods)

    if startup_hooks:
        # FIXME: there should be an error or a warning if startup defined in
        # mods beyond the first one
        startup_hooks[0](global_context)

    if global_context.backend:
        global_context.backend.register_command_contexts(global_context)
    for command in find_command_hooks(mods):
        global_context.register_command(command.name, command)
    register_stuff(global_context)
    for cmd_name in global_context.command_names():
        register_options(global_context, cmd_name)

    if global_context.backend:
        global_context.backend.register_options_contexts(global_context)
    if option_hooks:
        # FIXME: there should be an error or a warning if shutdown defined in
        # mods beyond the first one
        option_hooks[0](global_context)

    # FIXME: this registered options for new commands registered in hook. It
    # should be made all in one place (hook and non-hook)
    for cmd_name in global_context.command_names(public_only=False):
        if not global_context.is_options_context_registered(cmd_name):
            register_options(global_context, cmd_name)

    for cmd_name in global_context.command_names():
        for hook in find_pre_hooks(mods, cmd_name):
            global_context.add_pre_hook(hook, cmd_name)
        for hook in find_post_hooks(mods, cmd_name):
            global_context.add_post_hook(hook, cmd_name)

    try:
        return _main(global_context, cached_package, popts, run_node, top_node, build_node)
    finally:
        if shutdown_hooks:
            shutdown_hooks[0](global_context)

def create_global_options_context():
    context = OptionsContext(usage="%prog [options] [cmd_name [cmd_options]]")
    context.add_option(Option("--version", "-v", dest="show_version", action="store_true",
                              help="Version"))
    context.add_option(Option("--full-version", dest="show_full_version", action="store_true",
                              help="Full version"))
    context.add_option(Option("--build-directory", dest="build_directory",
                              help="Build directory as relative path from cwd (default: '%default')"))
    context.add_option(Option("--bento-info", dest="bento_info",
                              help="Bento location as a relative path from cwd (default: '%default'). " \
                                   "The base name (without its component) must be 'bento.info("))
    context.add_option(Option("--disable-autoconfigure", dest="disable_autoconfigure",
                              action="store_true",
                              default=False,
                              help="""\
Do not automatically run configure before build. In this mode, the user is
expected to know what he is doing. This is mainly useful for developers, to
avoid running configure everytime (default: '%default')."""))
    context.add_option(Option("-h", "--help", dest="show_help", action="store_true",
                              help="Display help and exit"))
    context.parser.set_defaults(show_version=False, show_full_version=False, show_help=False,
                                build_directory="build", bento_info="bento.info")
    return context

def parse_global_options(context, argv):
    global_args, cmd_args = [], []
    for i, a in enumerate(argv):
        if a.startswith("-"):
            global_args.append(a)
        else:
            cmd_args = argv[i:]
            break

    cmd_name = None
    cmd_argv = None
    if cmd_args:
        cmd_name = cmd_args[0]
        cmd_argv = cmd_args[1:]

    o, a = context.parser.parse_args(global_args)
    show_usage = o.show_help
    build_directory = o.build_directory
    if not os.path.basename(o.bento_info) == BENTO_SCRIPT:
        context.parser.error("Invalid value for --bento-info: %r (basename should be %r)" % \
                             (o.bento_info, BENTO_SCRIPT))

    bento_info = o.bento_info
    show_version = o.show_version
    show_full_version = o.show_full_version

    global_options = GlobalOptions(cmd_name, cmd_argv, show_usage,
            build_directory, bento_info, show_version, show_full_version,
            o.disable_autoconfigure)
    return global_options

def _main(global_context, cached_package, popts, run_node, top_node, build_node):
    if popts.show_usage:
        context_klass = global_context.retrieve_command_context("help")
        cmd = global_context.retrieve_command('help')
        cmd.run(context_klass(global_context, [], global_context.retrieve_options_context('help'), None, None))
        return

    cmd_name = popts.cmd_name
    cmd_argv = popts.cmd_argv

    if not cmd_name:
        print("Type '%s help' for usage." % SCRIPT_NAME)
        return 1
    else:
        if not global_context.is_command_registered(cmd_name):
            raise bento.errors.UsageException("%s: Error: unknown command %r" % (SCRIPT_NAME, cmd_name))
        else:
            run_cmd(global_context, cached_package, cmd_name, cmd_argv, run_node, top_node, build_node)

def _get_package_user_flags(global_context, package_options, configure_argv):
    from bento.commands.configure import _get_flag_values

    p = global_context.retrieve_options_context("configure")
    o, a = p.parser.parse_args(configure_argv)
    flag_values = _get_flag_values(package_options.flag_options.keys(), o)

    return flag_values

def is_help_only(global_context, cmd_name, cmd_argv):
    p = global_context.retrieve_options_context(cmd_name)
    o, a = p.parser.parse_args(cmd_argv)
    return o.help is True

def get_running_package(global_context, cached_package, bento_info):
    """Return a PackageDescription instance after evaluation of the flags set
    up at configure time.

    Example
    -------
    For this bento.info::

        Name: foo

        Flag: bundle
            Description: foo
            Default: true

        Library:
            if flag(bundle):
                Packages: yeah

    if bentomaker was run as follows::

        bentomaker configure

    get_running_package will return a PackageDescription instance with
    package.packages set to ["yeah"], and if bentomaker was run as follows::

        bentomaker configure --bundle=false

    get_running_package will return a PackageDescription instance with
    package.packages set to [].
    """
    package_options = cached_package.get_options(bento_info)
    configure_argv = global_context.retrieve_command_argv("configure")
    flag_values = _get_package_user_flags(global_context, package_options, configure_argv)
    return cached_package.get_package(bento_info, flag_values)

def run_cmd(global_context, cached_package, cmd_name, cmd_argv, run_node, top_node, build_node):
    cmd = global_context.retrieve_command(cmd_name)

    # XXX: fix this special casing (commands which do not need a pkg instance)
    if cmd_name in ["help", "convert"]:
        options_context = global_context.retrieve_options_context(cmd_name)
        context_klass = global_context.retrieve_command_context(cmd_name)
        context = context_klass(global_context, cmd_argv, options_context, PackageDescription(), run_node)
        cmd.run(context)
        return

    if is_help_only(global_context, cmd_name, cmd_argv):
        options_context = global_context.retrieve_options_context(cmd_name)
        options_context.parser.print_help()
        return

    bento_info = top_node.find_node(BENTO_SCRIPT)
    if bento_info is None:
        raise bento.errors.UsageException("Error: no %s found !" % os.path.join(top_node.abspath(), BENTO_SCRIPT))

    running_package = get_running_package(global_context, cached_package, bento_info)
    run_with_dependencies(global_context, cmd_name, cmd_argv, run_node, top_node, running_package)

    global_context.save_command_argv(cmd_name, cmd_argv)
    global_context.store()

def noexc_main(argv=None):
    def _print_debug():
        if BENTOMAKER_DEBUG:
            tb = sys.exc_info()[2]
            traceback.print_tb(tb)
    def _print_error(msg):
        pprint('RED', msg)
        if not BENTOMAKER_DEBUG:
            pprint('RED', "(You can see the traceback by setting the " \
                          "BENTOMAKER_DEBUG=1 environment variable)")

    try:
        main(argv)
    except bento.errors.BentoError:
        _print_debug()
        e = extract_exception()
        _print_error(str(e))
        sys.exit(2)
    except Exception:
        msg = """\
%s: Error: %s crashed (uncaught exception %s: %s).
Please report this on bento issue tracker:
    http://github.com/cournape/bento/issues"""
        if not BENTOMAKER_DEBUG:
            msg += "\nYou can get a full traceback by setting BENTOMAKER_DEBUG=1"
        else:
            _print_debug()
        e = extract_exception()
        pprint('RED',  msg % (SCRIPT_NAME, SCRIPT_NAME, e.__class__, str(e)))
        sys.exit(1)

if __name__ == '__main__':
    noexc_main()



class CommandRegistry(object):
    def __init__(self):
        # command line name -> command class
        self._klasses = {}
        # command line name -> None for private commands
        self._privates = {}

    def register(self, name, cmd_klass, public=True):
        if name in self._klasses:
            raise ValueError("context for command %r already registered !" % name)
        else:
            self._klasses[name] = cmd_klass
            if not public:
                self._privates[name] = None

    def retrieve(self, name):
        cmd_klass = self._klasses.get(name, None)
        if cmd_klass is None:
            raise ValueError("No command class registered for name %r" % name)
        else:
            return cmd_klass

    def is_registered(self, name):
        return name in self._klasses

    def command_names(self):
        return self._klasses.keys()

    def public_command_names(self):
        return [k for k in self._klasses.keys() if not k in self._privates]

class ContextRegistry(object):
    def __init__(self, default=None):
        self._contexts = {}
        self.set_default(default)

    def set_default(self, default):
        self._default = default

    def is_registered(self, cmd_name):
        return cmd_name in self._contexts

    def register(self, cmd_name, context):
        if cmd_name in self._contexts:
            raise ValueError("context for command %r already registered !" % cmd_name)
        else:
            self._contexts[cmd_name] = context

    def retrieve(self, cmd_name):
        context = self._contexts.get(cmd_name, None)
        if context is None:
            if self._default is None:
                raise ValueError("No context registered for command %r" % cmd_name)
            else:
                return self._default
        else:
            return context

class OptionsRegistry(object):
    """Registry for command -> option context"""
    def __init__(self):
        # command line name -> context *instance*
        self._contexts = {}

    def register(self, cmd_name, options_context):
        if cmd_name in self._contexts:
            raise ValueError("options context for command %r already registered !" % cmd_name)
        else:
            self._contexts[cmd_name] = options_context

    def is_registered(self, cmd_name):
        return cmd_name in self._contexts

    def retrieve(self, cmd_name):
        options_context = self._contexts.get(cmd_name, None)
        if options_context is None:
            raise ValueError("No options context registered for cmd_name %r" % cmd_name)
        else:
            return options_context



from bento.compat.api.moves \
    import \
        unittest

from bento.backends.utils \
    import \
        load_backend

class TestLoadBackend(unittest.TestCase):
    def test_simple(self):
        for backend in ("Distutils", "Yaku"):
            load_backend(backend)



import os
import shutil
import tempfile
import zipfile

import os.path as op

import mock

import bento.commands.build_wininst

from bento.commands.build_wininst \
    import \
        create_wininst
from bento.compat.api.moves \
    import \
        unittest
from bento.core.node \
    import \
        create_base_nodes
from bento.installed_package_description \
    import \
        InstalledPkgDescription

class TestWininstInfo(unittest.TestCase):
    def setUp(self):
        self.old_dir = None
        self.tmpdir = None

        self.old_dir = os.getcwd()
        self.tmpdir = tempfile.mkdtemp()

        try:
            self.top_node, self.build_node, self.run_node = \
                    create_base_nodes(self.tmpdir, op.join(self.tmpdir, "build"))
            os.chdir(self.tmpdir)
        except:
            shutil.rmtree(self.tmpdir)
            raise

    def tearDown(self):
        os.chdir(self.old_dir)
        shutil.rmtree(self.tmpdir)

    @mock.patch("bento.commands.build_wininst.create_exe", mock.MagicMock())
    def test_simple(self):
        """This just tests whether create_wininst runs at all and produces a zip-file."""
        ipackage = InstalledPkgDescription({}, {"name": "foo", "version": "1.0"}, {})
        create_wininst(ipackage, self.build_node, self.build_node, wininst="foo.exe", output_dir="dist")
        arcname = bento.commands.build_wininst.create_exe.call_args[0][1]
        fp = zipfile.ZipFile(arcname)
        try:
            fp.namelist()
        finally:
            fp.close()



from bento.commands.command_contexts \
    import \
        _RegistryBase
from bento.compat.api import moves

class Test_RegistryBase(moves.unittest.TestCase):
    def test_simple(self):
        registry = _RegistryBase()
        registry.register_category("dummy", lambda: 1)
        registry.register_callback("dummy", "dummy_func", lambda: 2)

        self.assertEqual(registry.callback("dummy", "dummy_func")(), 2)
        self.assertEqual(registry.callback("dummy", "non_existing_dummy_func")(), 1)

    def test_double_registration(self):
        registry = _RegistryBase()
        registry.register_category("dummy", lambda: 1)
        self.assertRaises(ValueError, lambda: registry.register_category("dummy", lambda: 2))

        self.assertEqual(registry.callback("dummy", "non_existing_dummy_func")(), 1)

    def test_missing_category(self):
        registry = _RegistryBase()
        self.assertRaises(ValueError, lambda: registry.register_callback("dummy", "dummy_func", lambda: 2))
        self.assertRaises(ValueError, lambda: registry.callback("dummy", "dummy_func"))

    def test_default_callback(self):
        registry = _RegistryBase()

        registry.register_category("dummy", lambda: 1)
        self.assertEqual(registry.default_callback("dummy"), 1)
        self.assertRaises(ValueError, lambda: registry.default_callback("non_existing_category"))



import sys
import traceback
import optparse

import os.path as op

from six.moves \
    import \
        cStringIO

from bento.utils.utils \
    import \
        pprint, extract_exception, comma_list_split
from bento.core.package \
    import \
        static_representation
from bento.commands.core \
    import \
        Command
from bento.errors \
    import \
        UsageException, ConvertionError
from bento.convert.core \
    import \
        detect_monkeys, monkey_patch, analyse_setup_py, build_pkg
from bento.convert.utils \
    import \
        whole_test

class ConvertCommand(Command):
    long_descr = """\
Purpose: convert a setup.py to an .info file
Usage:   bentomaker convert [OPTIONS] setup.py"""
    short_descr = "convert distutils/setuptools project to bento."
    common_options = Command.common_options + [
        optparse.Option("-t", help="TODO", default="automatic",
               dest="type"),
        optparse.Option("-o", "--output", help="output file",
               default="bento.info",
               dest="output_filename"),
        optparse.Option("-v", "--verbose", help="verbose run",
               action="store_true"),
        optparse.Option("--setup-arguments",
               help="arguments to give to setup" \
                    "For example, --setup-arguments=-q,-n,--with-speedup will " \
                    "call python setup.py -q -n --with-speedup",
               dest="setup_args")]

    def run(self, ctx):
        argv = ctx.command_argv
        p = ctx.options_context.parser
        o, a = p.parse_args(argv)
        if o.help:
            p.print_help()
            return
        if len(a) < 1:
            filename = "setup.py"
        else:
            filename = a[0]
        if not op.exists(filename):
            raise ValueError("file %s not found" % filename)

        output = o.output_filename
        if op.exists(output):
            raise UsageException("file %s exists, not overwritten" % output)

        if o.verbose:
            show_output = True
        else:
            show_output = False

        if o.setup_args:
            setup_args = comma_list_split(o.setup_args)
        else:
            setup_args = ["-q", "-n"]

        monkey_patch_mode = o.type

        convert_log = "convert.log"
        log = open(convert_log, "w")
        try:
            try:
                convert(ctx, filename, setup_args, monkey_patch_mode, o.verbose, output, log, show_output)
            except ConvertionError:
                raise
            except Exception:
                e = extract_exception()
                log.write("Error while converting - traceback:\n")
                tb = sys.exc_info()[2]
                traceback.print_tb(tb, file=log)
                msg = "Error while converting %s - you may look at %s for " \
                      "details (Original exception: %s %s)" 
                raise ConvertionError(msg % (filename, convert_log, type(e), str(e)))
        finally:
            log.flush()
            log.close()

def convert(ctx, filename, setup_args, monkey_patch_mode, verbose, output, log, show_output=True):
    if monkey_patch_mode == "automatic":
        try:
            if verbose:
                pprint("PINK",
                       "Catching monkey (this may take a while) ...")
            monkey_patch_mode = detect_monkeys(filename, show_output, log)
            if verbose:
                pprint("PINK", "Detected mode: %s" % monkey_patch_mode)
        except ValueError:
            e = extract_exception()
            raise UsageException("Error while detecting setup.py type " \
                                 "(original error: %s)" % str(e))

    monkey_patch(ctx.top_node, monkey_patch_mode, filename)
    dist, package_objects = analyse_setup_py(filename, setup_args)
    pkg, options = build_pkg(dist, package_objects, ctx.top_node)

    out = static_representation(pkg, options)
    if output == '-':
        for line in out.splitlines():
            pprint("YELLOW", line)
    else:
        fid = open(output, "w")
        try:
            fid.write(out)
        finally:
            fid.close()

class DetectTypeCommand(Command):
    long_descr = """\
Purpose: detect type of distutils extension used by given setup.py
Usage:   bentomaker detect_type [OPTIONS]."""
    short_descr = "detect extension type."
    common_options = Command.common_options + [
        optparse.Option("-i", "--input", help="TODO", default="setup.py", dest="setup_file"),
        optparse.Option("-v", "--verbose", help="verbose run", action="store_true")]

    def run(self, ctx):
        argv = ctx.command_argv
        p = ctx.options_context.parser
        o, a = p.parse_args(argv)
        if o.help:
            p.print_help()
            return
        verbose = o.verbose

        log = cStringIO()

        if verbose:
            print("=================================================================")
            print("Detecting used distutils extension(s) ... (This may take a while)")
        monkey_patch_mode = whole_test(o.setup_file, o.verbose, log)
        if verbose:
            print("Done !")
            print("=================================================================")
        print("Detected type: %r" % monkey_patch_mode)



import os
import sys

import bento.utils.path

def get_scheme(platform):
    # Whenever you add a variable in schemes, you should add one in
    # scheme_opts as well, otherwise the it will not be customizable from
    # configure.
    schemes = {
        'unix': {
            'destdir': bento.utils.path.find_root(sys.prefix),
            'prefix': sys.prefix,
            'eprefix': sys.exec_prefix,
            'bindir': '$eprefix/bin',
            'sbindir': '$eprefix/sbin',
            'libexecdir': '$eprefix/libexec',
            'sysconfdir': '$prefix/etc',
            'sharedstatedir': '$prefix/com',
            'localstatedir': '$prefix/var',
            'libdir': '$eprefix/lib',
            'includedir': '$prefix/include',
            'datarootdir': '$prefix/share',
            'datadir': '$datarootdir',
            'mandir': '$datarootdir/man',
            'infodir': '$datarootdir/info',
            'localedir': '$datarootdir/locale',
            'docdir': '$datarootdir/doc/$pkgname',
            'htmldir': '$docdir',
            'dvidir': '$docdir',
            'psdir': '$docdir',
            'pdfdir': '$docdir',
            'sitedir': '$libdir/python$py_version_short/site-packages',
            'pkgdatadir': '$datadir/$pkgname'
        },
        'win32': {
            'destdir': bento.utils.path.find_root(sys.prefix),
            'prefix': sys.prefix,
            'eprefix': r'$prefix',
            'bindir': r'$eprefix\Scripts',
            'sbindir': r'$eprefix\Scripts',
            'libexecdir': r'$eprefix\Scripts',
            'sysconfdir': r'$prefix\etc',
            'sharedstatedir': r'$prefix\com',
            'localstatedir': r'$prefix\var',
            'libdir': r'$eprefix\lib',
            'includedir': r'$prefix\include',
            'datarootdir': r'$prefix\share',
            'datadir': r'$datarootdir',
            'mandir': r'$datarootdir\man',
            'infodir': r'$datarootdir\info',
            'localedir': r'$datarootdir\locale',
            'docdir': r'$datarootdir\doc\$pkgname',
            'htmldir': r'$docdir',
            'dvidir': r'$docdir',
            'psdir': r'$docdir',
            'pdfdir': r'$docdir',
            'sitedir': r'$prefix\Lib\site-packages',
            'pkgdatadir': r'$datadir\$pkgname'
        }
    }

    schemes_opts = {
        'prefix': {'opts': ['--prefix'],
                   'help': 'install architecture-independent files '
                           'in PREFIX [%s]'},
        'eprefix': {'opts': ['--exec-prefix'],
                   'help': 'install architecture-dependent files '
                           'in EPREFIX [%s]',
                    'dest': 'eprefix'},
        'bindir': {'opts': ['--bindir'],
                   'help': 'user executables [%s]'},
        'sbindir': {'opts': ['--sbindir'],
                   'help': 'system admin executables [%s]'},
        'libexecdir': {'opts': ['--libexecdir'],
                       'help': 'program executables [%s]'},
        'sysconfdir': {'opts': ['--sysconfdir'],
                       'help': 'read-only single-machine data [%s]'},
        'sharedstatedir': {'opts': ['--sharedstatedir'],
                           'help': 'modifiable architecture-independent data [%s]'},
        'localstatedir': {'opts': ['--localstatedir'],
                          'help': 'modifiable single-machine data [%s]'},
        'libdir': {'opts': ['--libdir'],
                   'help': 'object code library [%s]'},
        'includedir': {'opts': ['--includedir'],
                   'help': 'C header files [%s]'},
        'datarootdir': {'opts': ['--datarootdir'],
                   'help': 'read-only arch.-independent data root [%s]'},
        'datadir': {'opts': ['--datadir'],
                   'help': 'read-only arch.-independent data [%s]'},
        'infodir': {'opts': ['--infodir'],
                   'help': 'info documentation [%s]'},
        'localedir': {'opts': ['--localedir'],
                   'help': 'locale-dependent files [%s]'},
        'mandir': {'opts': ['--mandir'],
                   'help': 'man documentation [%s]'},
        'docdir': {'opts': ['--docdir'],
                   'help': 'documentation root [%s]'},
        'htmldir': {'opts': ['--htmldir'],
                   'help': 'html documentation [%s]'},
        'dvidir': {'opts': ['--dvidir'],
                   'help': 'dvi documentation [%s]'},
        'psdir': {'opts': ['--psdir'],
                   'help': 'ps documentation [%s]'},
        'pdfdir': {'opts': ['--pdfdir'],
                   'help': 'pdf documentation [%s]'},
        'sitedir': {'opts': ['--sitedir'],
                    'help': 'python site-packages [%s]'},
        'pkgdatadir': {'opts': ['--pkgdatadir'],
                    'help': 'package-specific data dir [%s]'},
        'destdir': {'opts': ['--destdir'],
                    'help': 'alternate root to install to [%s]'}
    }
    if platform.startswith('win32'):
        pkg_platform = platform
    else:
        pkg_platform = 'unix'

    try:
        scheme = schemes[pkg_platform]
    except KeyError:
        raise ValueError("Platform %s not yet supported" % platform)

    scheme_opts = {}
    for k, v in schemes_opts.items():
        val = schemes_opts[k].copy()
        val['help'] = val['help'] % scheme[k]
        scheme_opts[k] = val

    return scheme, scheme_opts



import os
import warnings

from bento._config \
    import \
        IPKG_PATH
from bento.commands.core \
    import \
        Command, Option
from bento.commands.egg_utils \
    import \
        EggInfo, egg_filename
from bento.utils.utils import pprint, extract_exception
from bento.core \
    import \
        PackageMetadata
from bento.private.bytecode \
    import \
        bcompile, PyCompileError
from bento.installed_package_description \
    import \
        InstalledPkgDescription, iter_files

import bento.compat.api as compat
import bento.utils.path

class BuildEggCommand(Command):
    long_descr = """\
Purpose: build egg
Usage:   bentomaker build_egg [OPTIONS]"""
    short_descr = "build egg."
    common_options = Command.common_options \
                        + [Option("--output-dir",
                                  help="Output directory", default="dist"),
                           Option("--output-file",
                                  help="Output filename")]

    def run(self, ctx):
        argv = ctx.command_argv
        p = ctx.options_context.parser
        o, a = p.parse_args(argv)
        if o.help:
            p.print_help()
            return
        output_dir = o.output_dir
        output_file = o.output_file

        n = ctx.build_node.make_node(IPKG_PATH)
        ipkg = InstalledPkgDescription.from_file(n.abspath())
        build_egg(ipkg, ctx, ctx.build_node, output_dir, output_file)

def build_egg(ipkg, ctx, source_root, output_dir=None, output_file=None):
    meta = PackageMetadata.from_ipkg(ipkg)
    egg_info = EggInfo.from_ipkg(ipkg, ctx.build_node)

    # FIXME: fix egg name
    if output_dir is None:
        if output_file is None:
            egg = egg_filename(os.path.join("dist", meta.fullname))
        else:
            egg = os.path.join("dist", output_file)
    else:
        if output_file is None:
            egg = egg_filename(os.path.join(output_dir, meta.fullname))
        else:
            egg = os.path.join(output_dir, output_file)
    bento.utils.path.ensure_dir(egg)

    zid = compat.ZipFile(egg, "w", compat.ZIP_DEFLATED)
    try:
        ipkg.update_paths({"prefix": source_root.abspath(),
                           "eprefix": source_root.abspath(),
                           "sitedir": source_root.abspath()})
        for filename, cnt in egg_info.iter_meta(ctx.build_node):
            zid.writestr(os.path.join("EGG-INFO", filename), cnt)

        file_sections = ipkg.resolve_paths(source_root)
        for kind, source, target in iter_files(file_sections):
            if not kind in ["executables"]:
                zid.write(source.abspath(), target.path_from(source_root))

        pprint("PINK", "Byte-compiling ...")
        for kind, source, target in iter_files(file_sections):
            if kind in ["pythonfiles"]:
                try:
                    bytecode = bcompile(source.abspath())
                except PyCompileError:
                    e = extract_exception()
                    warnings.warn("Error byte-compiling %r" % source.abspath())
                else:
                    zid.writestr("%sc" % target.path_from(source_root), bcompile(source.abspath()))
    finally:
        zid.close()

    return



import os
import sys
import subprocess

if sys.version_info[0] < 3:
    from cPickle \
        import \
            load, dump, dumps
else:
    from pickle \
        import \
            load, dump, dumps

from yaku._config \
    import \
        DEFAULT_ENV, BUILD_CONFIG, BUILD_CACHE, CONFIG_CACHE, HOOK_DUMP, \
        _OUTPUT
from yaku.environment \
    import \
        Environment
from yaku.tools \
    import \
        import_tools
from yaku.utils \
    import \
        ensure_dir, rename, join_bytes
from yaku.errors \
    import \
        UnknownTask, ConfigurationFailure, TaskRunFailure, WindowsError
import yaku.node
import yaku.task_manager

def create_top_nodes(start_dir, build_dir):
    root = yaku.node.Node("", None)
    if not os.path.exists(build_dir):
        os.makedirs(build_dir)
    if not os.path.exists(start_dir):
        raise ValueError("%s does not exist ???" % start_dir)
    srcnode = root.find_dir(start_dir)
    bldnode = root.find_dir(build_dir)

    # FIXME
    class _FakeContext(object):
        pass
    yaku.node.Node.ctx = _FakeContext()
    yaku.node.Node.ctx.srcnode = srcnode
    yaku.node.Node.ctx.bldnode = bldnode

    return srcnode, bldnode

# XXX: yet another hack. We need to guarantee that a same file is
# represented by exactly the same node instance (defined as the same
# id) everywhere. Pickling does not guarantee that, so for the files
# mapping, we store the nodes as paths and pickle that. We recreate
# the nodes from there when we need to reload it.
def _hook_id_to_hook_path(hook_dict):
    # convert a hook dict indexed by id to a hook dict indexed by
    # paths (for storage)
    return dict([(k.srcpath(), v) for k, v in hook_dict.items()])

def _hook_path_to_hook_id(src_root, hook_dict):
    # convert a hook dict indexed by paths to a hook dict indexed by
    # id
    return dict([(src_root.find_resource(k), v) for \
                 k, v in hook_dict.items()])

class ConfigureContext(object):
    def __init__(self):
        self.env = Environment()
        self.tools = []
        self._tool_modules = {}
        self.builders = {}
        self.cache = {}
        self.conf_results = []
        self._configured = {}
        self._stdout_cache = {}
        self._cmd_cache = {}

        self.src_root = None
        self.bld_root = None
        self.path = None

    def load_tool(self, tool, tooldir=None):
        _t = import_tools([tool], tooldir)
        self.tools.append({"tool": tool, "tooldir": tooldir})
        mod = _t[tool]
        self._tool_modules[tool] = mod
        if hasattr(mod, "get_builder"):
            self.builders[tool] = mod.get_builder(self)
        return mod

    def use_tools(self, tools, tooldir=None):
        ret = {}
        for t in tools:
            if t in self._tool_modules:
                _t = self._tool_modules[t]
            else:
                _t = self.load_tool(t, tooldir)
            ret[t] = _t
        self.setup_tools()
        return ret

    def setup_tools(self):
        for builder in self.builders.values():
            if not builder.configured:
               builder.configure()
               self._configured[builder] = True

    def store(self):
        default_env = self.bld_root.make_node(DEFAULT_ENV)
        self.env.store(default_env.abspath())

        self.log.close()

        config_cache = self.bld_root.make_node(CONFIG_CACHE)
        out = []
        out.append(dumps(self.cache))
        out.append(dumps(self._stdout_cache))
        out.append(dumps(self._cmd_cache))
        config_cache.write(join_bytes(out), flags="wb")

        build_config = self.bld_root.make_node(BUILD_CONFIG)
        build_config.write("%r\n" % self.tools)

        hook_dump = self.bld_root.make_node(HOOK_DUMP)
        s = dumps({"extensions": yaku.task_manager.RULES_REGISTRY,
                   "files": _hook_id_to_hook_path(yaku.task_manager.FILES_REGISTRY)})
        hook_dump.write(s, flags="wb")

    def start_message(self, msg):
        _OUTPUT.write(msg + "... ")
        self.log.write("=" * 79 + "\n")
        self.log.write("%s\n" % msg)

    def end_message(self, msg):
        _OUTPUT.write("%s\n" % msg)

    def fail_configuration(self, msg):
        msg = "%s\nPlease look at the configuration log %r" % (msg, self.log.name)
        self.log.flush()
        raise ConfigurationFailure(msg)

    def set_cmd_cache(self, task, cmd):
        self._cmd_cache[task.get_uid()] = cmd[:]

    def get_cmd(self, task):
        tid = task.get_uid()
        try:
            return self._cmd_cache[tid]
        except KeyError:
            raise UnknownTask

    def set_stdout_cache(self, task, stdout):
        self._stdout_cache[task.get_uid()] = stdout

    def get_stdout(self, task):
        tid = task.get_uid()
        try:
            return self._stdout_cache[tid]
        except KeyError:
            raise UnknownTask

    def try_command(self, cmd, cwd=None, env=None, kw=None):
        if cwd is None:
            cwd = self.bld_root.abspath()
        if kw is None:
            kw = {}
        if env is not None:
            kw["env"] = env

        def log_failure(succeed, explanation):
            if succeed:
                self.log.write("---> Succeeded !\n")
            else:
                self.log.write("---> Failure !\n")
                self.log.write("~~~~~~~~~~~~~~\n")
                self.log.write(explanation)
                self.log.write("~~~~~~~~~~~~~~\n")
            self.log.write("Command was:\n")
            self.log.write("%s\n" % cmd)
            self.log.write("\n")

        try:
            p = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                    stderr=subprocess.STDOUT, cwd=cwd, **kw)
            stdout = p.communicate()[0].decode("utf-8")
            log_failure(p.returncode == 0, stdout)
            if p.returncode:
                return False
            return True
        except OSError:
            e = get_exception()
            log_failure(False, str(e))
        except WindowsError:
            e = get_exception()
            log_failure(False, str(e))
        return False

def load_tools(self, fid):
    tools = eval(fid.read())
    for t in tools:
        _t = import_tools([t["tool"]], t["tooldir"])
        tool_name = t["tool"]
        tool_mod = _t[tool_name]
        if hasattr(tool_mod, "get_builder"):
            self.builders[tool_name] = tool_mod.get_builder(self)
        # XXX: this is ugly - need to rethinkg tool
        # initialization/configuration
        if hasattr(tool_mod, "init"):
            tool_mod.init()
    self.tools = tools

class BuildContext(object):
    def __init__(self):
        self.env = Environment()
        self.tools = []
        self.cache = {}
        self.builders = {}
        self.tasks = []

    def load(self, src_path=None, build_path="build"):
        if src_path is None:
            src_path = os.getcwd()
        src_path = os.path.abspath(src_path)
        build_path = os.path.abspath(os.path.join(os.getcwd(), build_path))
        if not os.path.exists(build_path):
            raise IOError("%s not found (did you use different build_path for configure and build contexts ?)" \
                          % build_path)

        srcnode, bldnode = create_top_nodes(src_path, build_path)
        self.src_root = srcnode
        self.bld_root = bldnode
        self.path = srcnode

        self.env = Environment()
        default_env = bldnode.find_node(DEFAULT_ENV)
        if default_env:
            self.env.load(default_env.abspath())
        if not os.path.abspath(self.env["BLDDIR"]) == bldnode.abspath():
            raise ValueError("Gne ?")

        build_config = bldnode.find_node(BUILD_CONFIG)
        if build_config is None:
            raise IOError("Did not find %r in %r" % (BUILD_CONFIG, bldnode.abspath()))
        else:
            f = open(build_config.abspath())
            try:
                load_tools(self, f)
            finally:
                f.close()

        build_cache = bldnode.find_node(BUILD_CACHE)
        if build_cache is not None:
            fid = open(build_cache.abspath(), "rb")
            try:
                self.cache = load(fid)
            finally:
                fid.close()
        else:
            self.cache = {}

        hook_dump = bldnode.find_node(HOOK_DUMP)
        fid = open(hook_dump.abspath(), "rb")
        try:
            data = load(fid)
            yaku.task_manager.RULES_REGISTRY = data["extensions"]
            yaku.task_manager.FILES_REGISTRY = _hook_path_to_hook_id(srcnode, data["files"])
        finally:
            fid.close()

    def store(self):
        build_cache = self.bld_root.make_node(BUILD_CACHE)
        tmp_fid = open(build_cache.abspath() + ".tmp", "wb")
        try:
            dump(self.cache, tmp_fid)
        finally:
            tmp_fid.close()
        rename(build_cache.abspath() + ".tmp", build_cache.abspath())

    def set_stdout_cache(self, task, stdout):
        pass

    def set_cmd_cache(self, task, stdout):
        pass

def myopen(filename, mode="r"):
    if "w" in mode:
        ensure_dir(filename)
    return open(filename, mode)

def get_cfg(src_path=None, build_path="build"):
    ctx = ConfigureContext()
    config_cache = os.path.join(build_path, CONFIG_CACHE)
    if os.path.exists(config_cache):
        fid = open(config_cache, "rb")
        try:
            ctx.cache = load(fid)
            ctx._stdout_cache = load(fid)
            ctx._cmd_cache = load(fid)
        finally:
            fid.close()

    # XXX: how to reload existing environment ?
    env = Environment()
    if not "BLDDIR" in env:
        env["BLDDIR"] = build_path
    # FIXME: nothing to do here
    env["VERBOSE"] = False
    if "-v" in sys.argv:
        env["VERBOSE"] = True
    # Keep this as is - we do want a dictionary for 'serialization', and python
    # 3 os.environ is an object instead of a dict
    env["ENV"] = dict([(k, v) for k, v in os.environ.items()])

    if src_path is None:
        src_path = os.getcwd()
    srcnode, bldnode = create_top_nodes(
            os.path.abspath(src_path),
            os.path.abspath(env["BLDDIR"]))
    ctx.src_root = srcnode
    ctx.bld_root = bldnode
    # src_root and bld_root never change, but path may. All source nodes are
    # created relatively to path (kinda 'virtual' cwd)
    ctx.path = srcnode

    ctx.env = env
    ctx.log = myopen(os.path.join(env["BLDDIR"], "config.log"), "w")
    return ctx

def get_bld(src_path=None, build_path="build"):
    ctx = BuildContext()
    ctx.load(src_path=src_path, build_path=build_path)

    return ctx



from os.path \
    import \
        abspath, commonprefix, join

# strings representing various path-related bits and pieces
curdir = '.'
pardir = '..'
sep = '/'

def relpath(path, start=curdir):
    """Return a relative version of a path"""

    if not path:
        raise ValueError("no path specified")

    start_list = abspath(start).split(sep)
    path_list = abspath(path).split(sep)

    # Work out how much of the filepath is shared by start and path.
    i = len(commonprefix([start_list, path_list]))

    rel_list = [pardir] * (len(start_list)-i) + path_list[i:]
    if not rel_list:
        return curdir
    return join(*rel_list)



import yaku.tools

from yaku.task \
    import \
        task_factory
from yaku.task_manager \
    import \
        extension, TaskGen

def render(task):
    import jinja2
    content = task.inputs[0].read()
    template = jinja2.Template(content)
    task.outputs[0].write(template.render(**task.env["SUBST_DICT"]))

@extension(".in")
def template_task(task_gen, node):
    out = node.change_ext("")
    target = node.parent.declare(out.name)
    task = task_factory("subst")(inputs=[node], outputs=[target], func=render)
    task.env_vars = ["SUBST_DICT"]
    task.env = task_gen.env
    return [task]

class TemplateBuilder(yaku.tools.Builder):
    def render(self, sources, vars=None):
        if vars is None:
            self.env["SUBST_DICT"]  = {}
        else:
            self.env["SUBST_DICT"]  = vars
        return self._task_gen_factory("subst", "noname", sources, None)

def get_builder(context):
    return TemplateBuilder(context)



import sys
import os
import shutil

from bento.installed_package_description \
    import \
        InstalledSection
from bento.errors \
    import \
        CommandExecutionFailure
from bento.utils.utils \
    import \
        cpu_count, extract_exception
import bento.errors

import yaku.task_manager
import yaku.context
import yaku.scheduler
import yaku.errors

def build_extension(bld, extension, env=None):
    builder = bld.builders["pyext"]
    try:
        if env is None:
            env = {"PYEXT_CPPPATH": extension.include_dirs}
        else:
            val = env.get("PYEXT_CPPPATH", [])
            val.extend(extension.include_dirs)
        tasks = builder.extension(extension.name, extension.sources, env)
        if len(tasks) > 1:
            outputs = tasks[0].gen.outputs
        else:
            outputs = []
        return [n.bldpath() for n in outputs]
    except RuntimeError:
        e = extract_exception()
        msg = "Building extension %s failed: %s" % \
              (extension.name, str(e))
        raise CommandExecutionFailure(msg)

def build_compiled_library(bld, clib, env=None):
    builder = bld.builders["ctasks"]
    try:
        for p in clib.include_dirs:
            builder.env["CPPPATH"].insert(0, p)
        outputs = builder.static_library(clib.name, clib.sources, env)
        return [n.bldpath() for n in outputs]
    except RuntimeError:
        e = extract_exception()
        msg = "Building library %s failed: %s" % (clib.name, str(e))
        raise CommandExecutionFailure(msg)



"""
NamedTemporaryFile with delete option for python 2.4/2.5
"""
import os as _os
from tempfile \
    import \
        gettempdir, template

# Importing private stuff is ok here as this is for compat, and won't change
# (checked that python 2.4.4 has those values, has not checked for 2.5.*, but
# 2.6.4 define those to same values...
from tempfile \
    import \
        _text_openflags, _bin_openflags, _mkstemp_inner

# Code below taken verbatim from python 2.6.4
class _TemporaryFileWrapper:
    """Temporary file wrapper

    This class provides a wrapper around files opened for
    temporary use.  In particular, it seeks to automatically
    remove the file when it is no longer needed.
    """

    def __init__(self, file, name, delete=True):
        self.file = file
        self.name = name
        self.close_called = False
        self.delete = delete

    def __getattr__(self, name):
        # Attribute lookups are delegated to the underlying file
        # and cached for non-numeric results
        # (i.e. methods are cached, closed and friends are not)
        file = self.__dict__['file']
        a = getattr(file, name)
        if not issubclass(type(a), type(0)):
            setattr(self, name, a)
        return a

    # The underlying __enter__ method returns the wrong object
    # (self.file) so override it to return the wrapper
    def __enter__(self):
        self.file.__enter__()
        return self

    # NT provides delete-on-close as a primitive, so we don't need
    # the wrapper to do anything special.  We still use it so that
    # file.name is useful (i.e. not "(fdopen)") with NamedTemporaryFile.
    if _os.name != 'nt':
        # Cache the unlinker so we don't get spurious errors at
        # shutdown when the module-level "os" is None'd out.  Note
        # that this must be referenced as self.unlink, because the
        # name TemporaryFileWrapper may also get None'd out before
        # __del__ is called.
        unlink = _os.unlink

        def close(self):
            if not self.close_called:
                self.close_called = True
                self.file.close()
                if self.delete:
                    self.unlink(self.name)

        def __del__(self):
            self.close()

        # Need to trap __exit__ as well to ensure the file gets
        # deleted when used in a with statement
        def __exit__(self, exc, value, tb):
            result = self.file.__exit__(exc, value, tb)
            self.close()
            return result


def NamedTemporaryFile(mode='w+b', bufsize=-1, suffix="",
                       prefix=template, dir=None, delete=True):
    """Create and return a temporary file.
    Arguments:
    'prefix', 'suffix', 'dir' -- as for mkstemp.
    'mode' -- the mode argument to os.fdopen (default "w+b").
    'bufsize' -- the buffer size argument to os.fdopen (default -1).
    'delete' -- whether the file is deleted on close (default True).
    The file is created as mkstemp() would do it.

    Returns an object with a file-like interface; the name of the file
    is accessible as file.name.  The file will be automatically deleted
    when it is closed unless the 'delete' argument is set to False.
    """

    if dir is None:
        dir = gettempdir()

    if 'b' in mode:
        flags = _bin_openflags
    else:
        flags = _text_openflags

    # Setting O_TEMPORARY in the flags causes the OS to delete
    # the file when it is closed.  This is only supported by Windows.
    if _os.name == 'nt' and delete:
        flags |= _os.O_TEMPORARY

    (fd, name) = _mkstemp_inner(dir, prefix, suffix, flags)
    file = _os.fdopen(fd, mode, bufsize)
    return _TemporaryFileWrapper(file, name, delete)




import errno
import shutil

from bento.utils.utils \
    import \
        extract_exception
from bento.compat.api \
    import \
        rename as _rename

def rename(source, target):
    try:
        _rename(source, target)
    except OSError:
        e = extract_exception()
        if e.errno == errno.EXDEV:
            shutil.move(source, target)
        else:
            raise




import os
import shutil
import tempfile

import os.path as op

import mock

import bento.errors
import six

from bento.core.package \
    import \
        PackageDescription
from bento.pypi.register_utils \
    import \
        PyPIConfig
from bento.pypi.upload_utils \
    import \
        build_upload_post_data, build_request, upload

from bento.compat.api import moves

from six \
    import \
        PY3

if PY3:
    from urllib.request \
        import \
            HTTPPasswordMgr, urlparse, HTTPError, URLError
else:
    from urllib2 \
        import \
            Request, urlparse, HTTPError, URLError

# FIXME: there has to be a simpler way to do this
class MockedResult(object):
    def __init__(self, code, msg):
        self._code = code
        self.msg = msg

    def getcode(self):
        return self._code

#def raise_error_factory(url=None, code=200, msg=""):
#    if url is None:
#        url = "http://example.com"
#    def f(request):
#        raise urllib2.HTTPError(url, code, msg, {}, six.moves.StringIO())
#    return f

def my_urlopen_factory(exception):
    def my_urlopen(request):
        raise exception
    return my_urlopen

class TestUpload(moves.unittest.TestCase):
    def setUp(self):
        self.package = PackageDescription.from_string("""\
Name: foo
""")
        self.cwd = tempfile.mkdtemp()
        try:
            self.old_cwd = os.getcwd()
            os.chdir(self.cwd)

            filename = op.join(self.cwd, "foo.bin")
            fp = open(filename, "wb")
            try:
                fp.write(six.b("garbage"))
            finally:
                fp.close()

        except:
            shutil.rmtree(self.cwd)
            raise

    def tearDown(self):
        os.chdir(self.old_cwd)
        shutil.rmtree(self.cwd)

    def test_upload_post_data(self):
        post_data = build_upload_post_data("foo.bin", "bdist_dumb", self.package)
        self.assertEqual(post_data[":action"], "file_upload")
        self.assertEqual(post_data["content"], ("foo.bin", six.b("garbage")))

    def test_signing(self):
        self.assertRaises(NotImplementedError, build_upload_post_data, "foo.bin", "bdist_dumb", self.package, True)

    def test_build_request(self):
        repository = "http://localhost"
        post_data = build_upload_post_data("foo.bin", "bdist_dumb", self.package)
        request = build_request(repository, post_data, "dummy_auth")
        r_headers = {
                "Content-type": six.b("multipart/form-data; boundary=--------------GHSKFJDLGDS7543FJKLFHRE75642756743254"),
                "Content-length": "2238",
                "Authorization": "dummy_auth"}
        self.assertEqual(request.headers, r_headers)

    @mock.patch("bento.pypi.upload_utils.urlopen", lambda request: MockedResult(200, ""))
    def test_upload(self):
        config = PyPIConfig("john", "password", repository="http://localhost")
        upload("foo.bin", "bdist_dumb", self.package, config)

    @mock.patch("bento.pypi.upload_utils.urlopen", my_urlopen_factory(
        HTTPError("", 404, "url not found", {}, six.moves.StringIO())))
    def test_upload_error_404(self):
        config = PyPIConfig("john", "password", repository="http://localhost")
        self.assertRaises(bento.errors.PyPIError, upload, "foo.bin", "bdist_dumb", self.package, config)

    @mock.patch("bento.pypi.upload_utils.urlopen", my_urlopen_factory(URLError("dummy")))
    def test_upload_error_no_host(self):
        config = PyPIConfig("john", "password", repository="http://llocalhost")
        self.assertRaises(URLError, upload, "foo.bin", "bdist_dumb", self.package, config)

    @mock.patch("bento.pypi.upload_utils.urlopen", lambda request: MockedResult(200, ""))
    def test_upload_auth(self):
        config = PyPIConfig("john", "password", repository="http://localhost")
        self.assertRaises(NotImplementedError, upload, "foo.bin", "bdist_dumb", self.package, config, True)



import yaku.utils

def setup(ctx):
    env = ctx.env

    ctx.env["CC"] = ["clang"]
    ctx.env["CC_TGT_F"] = ["-c", "-o"]
    ctx.env["CC_SRC_F"] = []
    ctx.env["CFLAGS"] = []
    ctx.env["DEFINES"] = []
    ctx.env["LINK"] = ["clang"]
    ctx.env["LINKFLAGS"] = []
    ctx.env["LINK_TGT_F"] = ["-o"]
    ctx.env["LINK_SRC_F"] = []
    ctx.env["SHAREDLIB_FMT"] = "lib%s.so"
    ctx.env["SHLINK"] = ["clang"]
    ctx.env["SHLINKFLAGS"] = []
    ctx.env["SHLINK_TGT_F"] = ["-o"]
    ctx.env["SHLINK_SRC_F"] = []
    ctx.env["MODLINK"] = ["clang", "-bundle", "-undefined", "dynamic_lookup"]
    ctx.env["MODLINKFLAGS"] = []
    ctx.env["MODLINK_TGT_F"] = ["-o"]
    ctx.env["MODLINK_SRC_F"] = []
    ctx.env["CPPPATH"] = []
    ctx.env["CPPPATH_FMT"] = "-I%s"
    ctx.env["LIBDIR"] = []
    ctx.env["LIBS"] = []
    ctx.env["LIB_FMT"] = "-l%s"
    ctx.env["LIBDIR_FMT"] = "-L%s"

    ctx.env["CC_OBJECT_FMT"] = "%s.o"
    ctx.env["PROGRAM_FMT"] = "%s"

    ctx.env["CXX"] = ["clang"]
    ctx.env["CXX_TGT_F"] = ["-c", "-o"]
    ctx.env["CXX_SRC_F"] = []
    ctx.env["CXXFLAGS"] = []
    ctx.env["CXXLINK"] = ["clang"]
    ctx.env["CXXLINKFLAGS"] = []
    ctx.env["CXXLINK_TGT_F"] = ["-o"]
    ctx.env["CXXLINK_SRC_F"] = []
    ctx.env["CXXSHLINK"] = ["clang"]
    ctx.env["CXXSHLINKFLAGS"] = []
    ctx.env["CXXSHLINK_TGT_F"] = ["-o"]
    ctx.env["CXXSHLINK_SRC_F"] = []
    ctx.env["CPPPATH"] = []
    ctx.env["CPPPATH_FMT"] = "-I%s"
    ctx.env["LIBDIR"] = []
    ctx.env["LIBS"] = []
    ctx.env["FRAMEWORKS"] = []
    ctx.env["LIB_FMT"] = "-l%s"
    ctx.env["LIBDIR_FMT"] = "-L%s"

    ctx.env["CXX_OBJECT_FMT"] = "%s.o"
    ctx.env["PROGRAM_FMT"] = "%s"

def detect(ctx):
    if yaku.utils.find_program("clang") is None:
        return False
    else:
        return True



import warnings

try:
    from unittest.case import _ExpectedFailure as ExpectedFailure, _UnexpectedSuccess as UnexpectedSuccess
except ImportError:
    from unittest2.case import _ExpectedFailure as ExpectedFailure, _UnexpectedSuccess as UnexpectedSuccess

def install_proxy():
    import nose.proxy

    class MyResultProxy(nose.proxy.ResultProxy):
        def addExpectedFailure(self, test, err):
            #from nose.plugins.expected import ExpectedFailure
            self.assertMyTest(test)
            plugins = self.plugins
            plugins.addError(self.test, (ExpectedFailure, err, None))
            addExpectedFailure = getattr(self.result, "addExpectedFailure", None)
            if addExpectedFailure:
                self.result.addExpectedFailure(self.test, self._prepareErr(err))
            else:
                warnings.warn("TestResult has no addExpectedFailure method, reporting as passes",
                              RuntimeWarning)
                self.result.addSuccess(self)

        def addUnexpectedSuccess(self, test):
            #from nose.plugins.expected import UnexpectedSuccess
            self.assertMyTest(test)
            plugins = self.plugins
            plugins.addError(self.test, (UnexpectedSuccess, None, None))
            self.result.addUnexpectedSuccess(self.test)
            if self.config.stopOnError:
                self.shouldStop = True
    nose.proxy.ResultProxy = MyResultProxy

def install_result():
    import nose.result

    class MyTextTestResult(nose.result.TextTestResult):
        def addExpectedFailure(self, test, err):
            # 2.7 expected failure compat
            if ExpectedFailure in self.errorClasses:
                storage, label, isfail = self.errorClasses[ExpectedFailure]
                storage.append((test, self._exc_info_to_string(err, test)))
                self.printLabel(label, (ExpectedFailure, '', None))

        def addUnexpectedSuccess(self, test):
            # 2.7 unexpected success compat
            if UnexpectedSuccess in self.errorClasses:
                storage, label, isfail = self.errorClasses[UnexpectedSuccess]
                storage.append((test, 'This test was marked as an expected '
                    'failure, but it succeeded.'))
                self.printLabel(label, (UnexpectedSuccess, '', None))
    nose.result.TextTestResult = MyTextTestResult



import yaku.utils

def setup(ctx):
    env = ctx.env

    ctx.env.update(
       {"F77": ["gfortran"],
        "F77_LINK": ["gfortran"],
        "F77FLAGS": ["-W", "-g"],
        "F77_TGT_F": ["-o"],
        "F77_SRC_F": ["-c"],
        "F77_LINKFLAGS": [],
        "F77_LINK_TGT_F": ["-o"],
        "F77_LINK_SRC_F": [],
        "F77_OBJECT_FMT": "%s.o",
        "F77_PROGRAM_FMT": "%s"})

def detect(ctx):
    if yaku.utils.find_program("gfortran") is None:
        return False
    else:
        return True



import os

import bento.testing.bentos

from bento.core.package \
    import \
        PackageDescription
from bento.installed_package_description \
    import \
        InstalledSection, ipkg_meta_from_pkg

# FIXME: use correct install path instead of python package hack
BENTOS_DIR = os.path.dirname(bento.testing.bentos.__file__)
SPHINX_META = os.path.join(BENTOS_DIR, "sphinx_meta.info")

SPHINX_META_PKG = PackageDescription.from_file(SPHINX_META)

def create_simple_ipkg_args(top_node):
    files = ["scripts/foo.py", "scripts/bar.py"]
    srcdir = "source"

    nodes = [top_node.make_node(os.path.join(srcdir, f)) for f in files]
    for n in nodes:
        n.parent.mkdir()
        n.write("")
    section = InstalledSection.from_source_target_directories("pythonfiles",
                    "section1", os.path.join("$_srcrootdir", srcdir), "$prefix/target", files)
    sections = {"pythonfiles": {"section1": section}}

    meta = ipkg_meta_from_pkg(SPHINX_META_PKG)
    return meta, sections, nodes




import sys
import warnings

from bento.compat.api.moves \
    import \
        unittest
from bento.parser.parser \
    import \
        parse
from bento.parser.nodes \
    import \
        ast_walk
from bento.parser.visitor \
    import \
        Dispatcher

def parse_and_analyse(data):
    p = parse(data)
    dispatcher = Dispatcher()
    res = ast_walk(p, dispatcher)

    return res

def _empty_description():
    d = {"libraries": {}, "path_options": {}, "flag_options": {},
         "data_files": {}, "extra_source_files": [], "executables": {},
         "hook_files": []}
    return d

def _empty_library():
    d = {"name": "default", "py_modules": [], "packages": [], "extensions": {},
         "build_requires": [], "install_requires": [], "compiled_libraries": {},
         "sub_directory": None}
    return d

class TestSimpleMeta(unittest.TestCase):
    def setUp(self):
        self.ref = _empty_description()

    def test_empty(self):
        data = ""
        self.assertEqual(parse_and_analyse(data), {})

    def test_name(self):
        data = """\
Name: foo
"""
        self.ref["name"] = "foo"
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_summary(self):
        data = """\
Summary: a few words.
"""
        self.ref.update({"summary": "a few words."})
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_author(self):
        data = """\
Author: John Doe
"""
        self.ref.update({"author": "John Doe"})
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_maintainer(self):
        data = """\
Maintainer: John Doe
"""
        self.assertEqual(parse_and_analyse(data)["maintainer"], "John Doe")

    def test_use_backends(self):
        data = """\
UseBackends: foo
"""
        self.assertEqual(parse_and_analyse(data)["use_backends"], ["foo"])

        data = """\
UseBackends: foo, bar
"""
        self.assertEqual(parse_and_analyse(data)["use_backends"], ["foo", "bar"])

    def test_hook(self):
        data = """\
HookFile: bscript
"""
        self.assertEqual(parse_and_analyse(data)["hook_files"], ["bscript"])

        data = """\
HookFile: bscript, bscript2
"""
        self.assertEqual(parse_and_analyse(data)["hook_files"], ["bscript", "bscript2"])

class TestDescription(unittest.TestCase):
    def setUp(self):
        self.ref = _empty_description()

    def test_simple_single_line(self):
        data = "Description: some simple description"

        self.ref.update({"description": "some simple description"})
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_simple_indented_block(self):
        data = """\
Description:
    some simple description
    on multiple
    lines.
"""
        self.ref["description"] = "some simple description\non multiple\nlines."
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_simple_indented_block2(self):
        data = """\
Description: some simple description
    on multiple
    lines.
"""
        self.ref["description"] = "some simple description\non multiple\nlines."
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_nested_indented_block3(self):
        data = """\
Description: some
    simple
        description
            on
    
    multiple

    lines.
"""

        self.ref["description"] = """some
simple
    description
        on

multiple

lines.\
"""
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_nested_indented_block2(self):
        last_indent = " " * 4
        data = """\
Description: some
    simple
        description
            on
%s
Name: foo
""" % last_indent

        self.ref["name"] = "foo"
        self.ref["description"] = """some
simple
    description
        on
%s
""" % last_indent
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_description_from_file(self):
        data = """\
DescriptionFromFile: foo.rst
"""
        ret = parse_and_analyse(data)
        self.assertEqual(ret["description_from_file"], "foo.rst")

    def test_meta_template_file(self):
        s = warnings.filters[:]
        warnings.filterwarnings("ignore")
        try:
            data = """\
MetaTemplateFile: foo.py.in
"""
            ret = parse_and_analyse(data)
            self.assertEqual(ret["meta_template_files"], ["foo.py.in"])
        finally:
            warnings.filters = s

    def test_meta_template_files(self):
        data = """\
MetaTemplateFile: foo.py.in
"""
        ret = parse_and_analyse(data)
        self.assertEqual(ret["meta_template_files"], ["foo.py.in"])

class TestLibrary(unittest.TestCase):
    def setUp(self):
        self.ref = _empty_description()
        self.ref["libraries"]["default"] = _empty_library()

    def test_empty(self):
        data = """\
Library:
"""

        #self.ref["libraries"] = {"default": {"name": "default"}}
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_sub_directory(self):
        self.maxDiff = None
        data = """\
Library:
    SubDirectory: lib
"""

        self.ref["libraries"]["default"]["sub_directory"] = "lib"
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_modules(self):
        data = """\
Library:
    Modules: foo.py
"""

        self.ref["libraries"]["default"].update({"py_modules": ["foo.py"]})
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_simple_extension(self):
        data = """\
Library:
    Extension: _foo
        Sources: foo.c
"""
        extension = {"name": "_foo", "sources": ["foo.c"]}
        self.ref["libraries"]["default"].update({"extensions": {"_foo": extension}})
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_extension_include_dirs(self):
        data = """\
Library:
    Extension: _foo
        Sources: foo.c
        IncludeDirs: foo
"""
        r_extension = {"name": "_foo", "sources": ["foo.c"], "include_dirs": ["foo"]}
        extension = parse_and_analyse(data)["libraries"]["default"]["extensions"]["_foo"]
        self.assertEqual(extension, r_extension)

    def test_double_sources_extension(self):
        data = """\
Library:
    Extension: _foo
        Sources: foo.c
        Sources: bar.c
"""
        self.assertRaises(ValueError, lambda: parse_and_analyse(data))

    def test_build_requires(self):
        data = """\
Library:
    BuildRequires: foo
"""
        self.ref["libraries"]["default"].update({"build_requires": ["foo"]})
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_install_requires(self):
        data = """\
Library:
    InstallRequires: foo
"""
        self.ref["libraries"]["default"].update({"install_requires": ["foo"]})
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_compiled_library_include_dirs(self):
        data = """\
Library:
    CompiledLibrary: _foo
        Sources: foo.c
        IncludeDirs: foo
"""
        r_compiled_library = {"name": "_foo", "sources": ["foo.c"], "include_dirs": ["foo"]}
        compiled_library = parse_and_analyse(data)["libraries"]["default"]["compiled_libraries"]["_foo"]
        self.assertEqual(compiled_library, r_compiled_library)

    def test_double_sources_compiled_library(self):
        data = """\
Library:
    CompiledLibrary: _foo
        Sources: foo.c
        Sources: bar.c
"""
        self.assertRaises(ValueError, lambda: parse_and_analyse(data))

class TestPath(unittest.TestCase):
    def test_simple(self):
        data = """\
Path: manpath
    Description: man path
    Default: /usr/share/man
"""

        descr = _empty_description()
        descr["path_options"] = {"manpath": {"name": "manpath",
                                             "default": "/usr/share/man",
                                             "description": "man path"}}
        self.assertEqual(parse_and_analyse(data), descr)

    def test_multiple_path_sections(self):
        data = """\
Path: manpath
    Description: man path
    Default: /usr/share/man

Path: varpath
    Description: var path
    Default: /var
"""

        data_sections = parse_and_analyse(data)["path_options"]
        r_data_sections = {
                "manpath": {"name": "manpath", "description": "man path", "default": "/usr/share/man"},
                "varpath": {"name": "varpath", "description": "var path", "default": "/var"},
        }
        self.assertEqual(data_sections, r_data_sections)

    def test_conditional_path_section(self):
        data_template = """\
Path: manpath
    Description: man path
    if %s:
        Default: /usr/share/woman
    else:
        Default: /usr/share/man
"""

        data_sections = parse_and_analyse(data_template % "true")["path_options"]
        r_data_sections = {
                "manpath": {"name": "manpath", "description": "man path", "default": "/usr/share/woman"},
        }
        self.assertEqual(data_sections, r_data_sections)

        data_sections = parse_and_analyse(data_template % "false")["path_options"]
        r_data_sections = {
                "manpath": {"name": "manpath", "description": "man path", "default": "/usr/share/man"},
        }
        self.assertEqual(data_sections, r_data_sections)

    def test_invalid_path_section1(self):
        data = """\
Path: manpath
    Description: description
"""
        self.assertRaises(ValueError, lambda: parse_and_analyse(data))

        data = """\
Path: manpath
    Default: /foo/bar
"""
        self.assertRaises(ValueError, lambda: parse_and_analyse(data))

class TestDataFiles(unittest.TestCase):
    def test_simple(self):
        data = """\
DataFiles: man1doc
    TargetDir: /usr/share/man/man1
    SourceDir: doc/man1
    Files: foo.1
"""

        ref = _empty_description()
        ref["data_files"] = {"man1doc": {"name": "man1doc",
                                         "target_dir": "/usr/share/man/man1",
                                         "source_dir": "doc/man1",
                                         "files": ["foo.1"]}}
        self.assertEqual(parse_and_analyse(data), ref)

class TestFlag(unittest.TestCase):
    def setUp(self):
        self.ref = _empty_description()

    def test_simple(self):
        data = """\
Flag: debug
    Description: debug flag
    Default: false
"""
        self.ref["flag_options"] = {"debug": {"name": "debug",
                                              "default": "false",
                                              "description": "debug flag"}}
        self.assertEqual(parse_and_analyse(data), self.ref)

class TestConditional(unittest.TestCase):
    def setUp(self):
        self.ref = _empty_description()
        self.ref["libraries"]["default"] = _empty_library()

    def test_literal(self):
        data = """\
Library:
    if true:
        Modules: foo.py, bar.py
    else:
        Modules:  fubar.py
"""

        self.ref["libraries"]["default"].update({"py_modules": ["foo.py", "bar.py"]})
        self.assertEqual(parse_and_analyse(data), self.ref)

        data = """\
Library:
    if false:
        Modules: foo.py, bar.py
    else:
        Modules:  fubar.py
"""
        self.ref["libraries"]["default"]["py_modules"] = ["fubar.py"]

        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_os(self):
        data = """\
Library:
    if os(%s):
        Modules: foo.py, bar.py
    else:
        Modules:  fubar.py
""" % sys.platform

        self.ref["libraries"]["default"].update({"py_modules": ["foo.py", "bar.py"]})
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_flag(self):
        data = """\
Flag: debug
    Default: true

Library:
    if flag(debug):
        Modules: foo.py, bar.py
    else:
        Modules:  fubar.py
"""

        self.ref["libraries"]["default"].update({"py_modules": ["foo.py", "bar.py"]})
        self.ref["flag_options"] = {"debug": {"default": "true",
                                              "name": "debug"}}
        self.assertEqual(parse_and_analyse(data), self.ref)

        data = """\
Flag: debug
    Default: false

Library:
    if flag(debug):
        Modules: foo.py, bar.py
    else:
        Modules:  fubar.py
"""

        self.ref["libraries"]["default"]["py_modules"] = ["fubar.py"]
        self.ref["flag_options"]["debug"] = {"default": "false", "name": "debug"}

        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_not_flag(self):
        data = """\
Flag: debug
    Default: true

Library:
    if not flag(debug):
        Modules: foo.py, bar.py
    else:
        Modules:  fubar.py
"""

        self.ref["libraries"]["default"].update({"py_modules": ["fubar.py"]})
        self.ref["flag_options"] = {"debug": {"default": "true",
                                              "name": "debug"}}
        self.assertEqual(parse_and_analyse(data), self.ref)

    def test_missing_flag(self):
        data = """\
Flag: debug
    Default: true

Library:
    if flag(ddebug):
        Modules: bar.py
"""
        self.assertRaises(ValueError, lambda: parse_and_analyse(data))

        data = """\
Flag: debug
    Default: true

Library:
    if not flag(ddebug):
        Modules: bar.py
"""
        self.assertRaises(ValueError, lambda: parse_and_analyse(data))

    def test_not(self):
        data = """\
Library:
    if not true:
        Modules: foo.py, bar.py
    else:
        Modules:  fubar.py
"""

        self.ref["libraries"]["default"].update({"py_modules": ["fubar.py"]})
        self.assertEqual(parse_and_analyse(data), self.ref)

class TestExecutable(unittest.TestCase):
    def setUp(self):
        self.ref = _empty_description()

    def test_modules(self):
        data = """\
Executable: foo
    Module: foo.bar
    Function: main
"""
        self.ref["executables"] = {"foo": {"module": "foo.bar",
                                           "function": "main",
                                           "name": "foo"}}
        self.assertEqual(parse_and_analyse(data), self.ref)



import sys
import os
import os.path as op

from six import moves

from bento.utils.utils \
    import \
        subst_vars, virtualenv_prefix
from bento.core.platforms import \
        get_scheme
from bento._config \
    import \
        CONFIGURED_STATE_DUMP, BENTO_SCRIPT

from bento.commands.core \
    import \
        Command, Option
from bento.errors \
    import \
        UsageException

def set_scheme_unix(scheme, options, package):
    # This mess is the simplest solution I can think of to support:
    #   - /usr/local as default for prefix while using debian/ubuntu changes
    #   (dist-packages instead of site-packages)
    #   - virtualenv support
    #   - arbitrary python install
    #
    # Cases to consider:
    #   - if prefix is given:
    #       - prefix and exec_prefix are customized following their options
    #   - if prefix is not given:
    #       - if under virtualenv: prefix and exec_prefix are set to their sys.* values
    #       - else on unix != darwin: using /usr/local
    #       - else on darwin: using sys values
    if options.prefix is not None:
        scheme["prefix"] = options.prefix
        if options.eprefix is None:
            scheme["eprefix"] = scheme["prefix"]
        else:
            scheme["eprefix"] = options.eprefix
    elif options.prefix is None and options.eprefix is not None:
        raise NotImplementedError("Customizing exec_prefix without " \
                                  "customizing prefix is not implemented yet")
    elif options.prefix is None and options.eprefix is None:
        venv_prefix = virtualenv_prefix()
        if venv_prefix is not None:
            scheme["prefix"] = scheme["eprefix"] = venv_prefix
        elif sys.platform == "darwin":
            scheme["prefix"] = op.normpath(sys.prefix)
            scheme["eprefix"] = op.normpath(sys.exec_prefix)
        else:
            # XXX: unix_local is an ubuntu/debian thing only ?
            from distutils.command.install import INSTALL_SCHEMES
            if "unix_local" in INSTALL_SCHEMES:
                dist_scheme = INSTALL_SCHEMES["unix_local"]
                # This madness is used to support ubuntu/debian customization
                prefix = "/usr/local"
                base = "/usr"
                py_version = sys.version.split()[0]
                py_version_short = py_version[0:3]
                dist_name = package.name
                v = {"base": base, "py_version_short": py_version_short, "dist_name": dist_name}

                scheme["prefix"] = scheme["eprefix"] = prefix
                scheme["sitedir"] = subst_vars(dist_scheme["purelib"], v)
                scheme["includedir"] = subst_vars(dist_scheme["headers"], v)
            else:
                scheme["prefix"] = scheme["eprefix"] = "/usr/local"

def set_scheme_win32(scheme, options, package):
    if options.prefix is not None:
        scheme["prefix"] = options.prefix
        if options.eprefix is None:
            scheme["eprefix"] = scheme["prefix"]
        else:
            scheme["eprefix"] = options.eprefix
    elif options.prefix is None and options.eprefix is not None:
        raise NotImplementedError("Customizing exec_prefix without " \
                                  "customizing prefix is not implemented yet")
    elif options.prefix is None and options.eprefix is None:
        venv_prefix = virtualenv_prefix()
        if venv_prefix is not None:
            scheme["prefix"] = scheme["eprefix"] = venv_prefix
        else:
            scheme["prefix"] = op.normpath(sys.prefix)
            scheme["eprefix"] = op.normpath(sys.exec_prefix)

def set_scheme_options(scheme, options, package):
    """Set path variables given in options in scheme dictionary."""
    for k in scheme:
        if hasattr(options, k):
            val = getattr(options, k)
            if val:
                if not os.path.isabs(val):
                    msg = "value given for path option '%s' " \
                          "should be an absolute path (value " \
                          "given was '%s')" % (k, val)
                    raise UsageException(msg)
                scheme[k] = val

    if os.name == "posix":
        set_scheme_unix(scheme, options, package)
    elif os.name == "nt":
        set_scheme_win32(scheme, options, package)
    else:
        raise NotImplementedError("OS %s not supported" % os.name)

    # XXX: define default somewhere and stick with it
    if options.prefix is not None and options.eprefix is None:
        scheme["eprefix"] = scheme["prefix"]

def get_flag_values(cmd, cmd_argv):
    """Return flag values from the command instance and its arguments.

    Assumes cmd is an instance of Configure."""
    o, a = cmd.options_context.parser.parse_args(cmd_argv)
    flag_values = _get_flag_values(cmd.flags, o)
    return flag_values

def _get_flag_values(flag_names, options):
    """Return flag values as defined by the options."""
    flag_values = {}
    for option_name in flag_names:
        flag_value = getattr(options, option_name, None)
        if flag_value is not None:
            if flag_value.lower() in ["true", "yes"]:
                flag_values[option_name] = True
            elif flag_value.lower() in ["false", "no"]:
                flag_values[option_name] = False
            else:
                msg = """Option %s expects a true or false argument"""
                raise UsageException(msg % "--%s" % option_name)
    return flag_values

def _setup_options_parser(options_context, package_options):
    """Setup the command options parser, merging standard options as well
    as custom options defined in the bento.info file, if any.
    """
    scheme, scheme_opts_d = get_scheme(sys.platform)

    p = options_context

    # Create default path options
    scheme_opts = {}
    for name, opt_d in scheme_opts_d.items():
        o = opt_d.pop("opts")
        opt = Option(*o, **opt_d)
        scheme_opts[name] = opt

    # Add custom path options (as defined in bento.info) to the path scheme
    for name, f in package_options.path_options.items():
        scheme_opts[name] = \
            Option('--%s' % f.name,
                   help='%s [%s]' % (f.description, f.default_value))

    p.add_group("installation_options", "Installation fine tuning")
    for opt in scheme_opts.values():
        p.add_option(opt, "installation_options")

    flag_opts = {}
    if package_options.flag_options:
        p.add_group("optional_features", "Optional features")
        for name, v in package_options.flag_options.items():
            flag_opts[name] = Option(
                    "--%s" % v.name,
                    help="%s [default=%s]" % (v.description, v.default_value))
            p.add_option(flag_opts[name], "optional_features")

class ConfigureCommand(Command):
    long_descr = """\
Purpose: configure the project
Usage: bentomaker configure [OPTIONS]"""
    short_descr = "configure the project."

    def __init__(self, *a, **kw):
        super(ConfigureCommand, self).__init__(*a, **kw)
        self.scheme = {}
        self.flags = []

    def register_options(self, options_context, package_options=None):
        _setup_options_parser(options_context, package_options)

        self.scheme.update(_compute_scheme(package_options))
        self.flags.extend(package_options.flag_options.keys())

    def run(self, ctx):
        bento_script = ctx.top_node.find_node(BENTO_SCRIPT)
        if bento_script is None:
            raise IOError("%s not found ?" % BENTO_SCRIPT)

        args = ctx.command_argv
        o, a = ctx.options_context.parser.parse_args(args)
        if o.help:
            ctx.options_context.parser.print_help()
            return

        set_scheme_options(self.scheme, o, ctx.pkg)

def _compute_scheme(package_options):
    """Compute path and flags-related options as defined in the script file(s)

    Parameters
    ----------
    package_options: PackageOptions
    """
    scheme, scheme_opts_d = get_scheme(sys.platform)

    # XXX: abstract away those, as it is copied from distutils
    scheme['py_version_short'] = ".".join(str(part) for part in sys.version_info[:2])
    scheme['pkgname'] = package_options.name

    for name, f in package_options.path_options.items():
        scheme[name] = f.default_value
    return scheme



# Most of the code below is shamelessly copied from the six library from
# Benjamin Peterson
import sys
import types

PY3 = sys.version_info[0] == 3

def _import_module(name):
    """Import module, returning the module after the last dot."""
    __import__(name)
    return sys.modules[name]

class _LazyDescr(object):

    def __init__(self, name):
        self.name = name

    def __get__(self, obj, tp):
        result = self._resolve()
        setattr(obj, self.name, result)
        # This is a bit ugly, but it avoids running this again.
        delattr(tp, self.name)
        return result

class MovedModule(_LazyDescr):

    def __init__(self, name, old, new=None):
        super(MovedModule, self).__init__(name)
        if PY3:
            if new is None:
                new = name
            self.mod = new
        else:
            self.mod = old

    def _resolve(self):
        return _import_module(self.mod)


class MovedAttribute(_LazyDescr):

    def __init__(self, name, old_mod, new_mod, old_attr=None, new_attr=None):
        super(MovedAttribute, self).__init__(name)
        if PY3:
            if new_mod is None:
                new_mod = name
            self.mod = new_mod
            if new_attr is None:
                if old_attr is None:
                    new_attr = name
                else:
                    new_attr = old_attr
            self.attr = new_attr
        else:
            self.mod = old_mod
            if old_attr is None:
                old_attr = name
            self.attr = old_attr

    def _resolve(self):
        module = _import_module(self.mod)
        return getattr(module, self.attr)

class _MovedItems(types.ModuleType):
    """Lazy loading of moved objects"""



WRAPPER_ASSIGNMENTS = ('__module__', '__name__', '__doc__')
WRAPPER_UPDATES = ('__dict__',)
def update_wrapper(wrapper,
                   wrapped,
                   assigned = WRAPPER_ASSIGNMENTS,
                   updated = WRAPPER_UPDATES):
    """Update a wrapper function to look like the wrapped function

       wrapper is the function to be updated
       wrapped is the original function
       assigned is a tuple naming the attributes assigned directly
       from the wrapped function to the wrapper function (defaults to
       functools.WRAPPER_ASSIGNMENTS)
       updated is a tuple naming the attributes of the wrapper that
       are updated with the corresponding attribute from the wrapped
       function (defaults to functools.WRAPPER_UPDATES)
    """
    for attr in assigned:
        setattr(wrapper, attr, getattr(wrapped, attr))
    for attr in updated:
        getattr(wrapper, attr).update(getattr(wrapped, attr, {}))
    # Return the wrapper so this can be used as a decorator via partial()
    return wrapper

def partial(func, *a, **kw):
    def f(*_a, **_kw):
        new_a = list(a) + list(_a)
        new_kw = kw
        new_kw.update(_kw)
        return func(*new_a, **new_kw)
    f = update_wrapper(f, func)
    return f




import os.path as op

from bento.conv \
    import \
        pkg_to_distutils_meta
from bento.utils.utils \
    import \
        extract_exception
from bento.errors \
    import \
        InvalidPyPIConfig

from six.moves \
    import \
        configparser, StringIO
from six \
    import \
        PY3

import six

if PY3:
    from urllib.request \
        import \
            Request, HTTPBasicAuthHandler, HTTPError, URLError, build_opener
else:
    from urllib2 \
        import \
            Request, HTTPBasicAuthHandler, HTTPError, URLError, build_opener

DEFAULT_REPOSITORY = 'http://pypi.python.org/pypi'
DEFAULT_REALM = 'pypi'

REALM = DEFAULT_REALM
REPOSITORY = 'http://testpypi.python.org/pypi'

_BOUNDARY = six.b('--------------GHSKFJDLGDS7543FJKLFHRE75642756743254')

def _read_new_format(config, repository):
    sections = config.sections()

    # let's get the list of servers
    index_servers = config.get('distutils', 'index-servers')
    _servers = [server.strip() for server in
                index_servers.split('\n')
                if server.strip() != '']
    if _servers == []:
        # nothing set, let's try to get the default pypi
        if 'pypi' in sections:
            _servers = ['pypi']
        else:
            # the file is not properly defined
            raise InvalidPyPIConfig("No index-servers section or pypi section")
    for server in _servers:
        current = PyPIConfig()
        current.username = config.get(server, 'username')
        current.server = server

        # optional params
        def _get_default(key, default):
            if config.has_option(server, key):
                return config.get(server, key)
            else:
                return default

        current.repository = _get_default("repository", DEFAULT_REPOSITORY)
        current.realm = _get_default("realm", DEFAULT_REALM)
        current.password = _get_default("password", None)

        if (current.server == repository or current.repository == repository):
            return current
    raise InvalidPyPIConfig("No section for repository %r found" % repository)

def _read_old_format(config):
    server = 'server-login'
    if config.has_option(server, 'repository'):
        repository = config.get(server, 'repository')
    else:
        repository = DEFAULT_REPOSITORY

    return PyPIConfig(username=config.get(server, 'username'),
            password=config.get(server, 'password'),
            repository=repository,
            server=server,
            realm=DEFAULT_REALM)

def read_pypirc(repository=DEFAULT_REPOSITORY):
    """Read the default .pypirc file.

    Returns a PyPIConfig instance if the default .pypirc can be found. Raises
    an IOError otherwise

    Parameters
    ----------
    repository: str
        repository to use
    """
    rc = op.join(op.expanduser('~'), '.pypirc')
    if op.exists(rc):
        fp = open(rc, "rt")
        try:
            return parse_pypirc(fp, repository)
        finally:
            fp.close()
    else:
        return IOError("Default pypirc config file not found: %r" % rc)

def parse_pypirc(fp, repository=DEFAULT_REPOSITORY):
    """Parse the given pypi config file.

    Returns a PyPIConfig instance if the file can be parsed.

    Parameters
    ----------
    fp: file-like object
        contains the content of the config file
    repository: str
        repository to look for
    """
    config = configparser.RawConfigParser()
    config.readfp(fp)
    sections = config.sections()
    if 'distutils' in sections:
        return _read_new_format(config, repository)
    elif 'server-login' in sections:
        return _read_old_format(config)
    else:
        msg = "Unrecognized format"
        if hasattr(fp, "name"):
            msg += " (for file %r)" % fp.name
        raise ValueError(msg)

class PyPIConfig(object):
    @classmethod
    def from_file(cls, fp=None, repository=DEFAULT_REPOSITORY):
        """Create a PyPIConfig instance from the give file for the give repository.

        Parameters
        ----------
        fp: file-like object or None
            If None, attemps to read the .pypirc file. Otherwise, must be a
            file-like object
        repository: str
            Repository to consider in the .pypirc file.
        """
        if fp is None:
            return read_pypirc(repository)
        else:
            return parse_pypirc(fp, repository)

    @classmethod
    def from_string(cls, s, repository=DEFAULT_REPOSITORY):
        return cls.from_file(StringIO(s), repository)

    def __init__(self, username=None, password=None, repository=None,
            server=None, realm=None):
        self.username = username
        self.password = password
        self.repository = repository
        self.realm = realm
        self.server = server

def encode_multipart(fields, files, boundary=None):
    """Prepare a multipart HTTP request.

    *fields* is a sequence of (name: str, value: str) elements for regular
    form fields, *files* is a sequence of (name: str, filename: str, value:
    bytes) elements for data to be uploaded as files.

    Returns (content_type: bytes, body: bytes) ready for httplib.HTTP.
    """
    # Taken from http://code.activestate.com/recipes/146306

    if boundary is None:
        boundary = _BOUNDARY
    elif not isinstance(boundary, str):
        raise TypeError('boundary must be str, not %r' % type(boundary))

    l = []
    for key, values in fields:
        # handle multiple entries for the same name
        if not isinstance(values, (tuple, list)):
            values = [values]

        for value in values:
            l.extend((
                six.b('--') + boundary,
                # XXX should encode to match packaging but it causes bugs
                ('Content-Disposition: form-data; name="%s"' % key).encode("utf-8"),
                six.b(''),
                value.encode("utf-8")))

    for key, filename, value in files:
        l.extend((
            six.b('--') + boundary,
            ('Content-Disposition: form-data; name="%s"; filename="%s"' %
             (key, filename)).encode("utf-8"),
            six.b(''),
            value))

    l.append(six.b('--') + boundary + six.b('--'))
    l.append(six.b(''))

    body = six.b('\r\n').join(l)
    content_type = six.b('multipart/form-data; boundary=') + boundary
    return content_type, body

def build_post_data(pkg, action):
    data = pkg_to_distutils_meta(pkg)
    data[":action"] = action
    return data

def post_to_server(post_data, config, auth=None):
    """Send the given post_data to the pypi server.

    Parameters
    ----------
    post_data: dict
        Usually the dict returned by build_post_data
    config: object
        A PyPIConfig instance
    auth: object or None
        HTTP authentification object.

    Returns
    -------
    code: int
        HTTP status code
    msg: str
        Message received back from the server
    """
    content_type, body = encode_multipart(post_data.items(), [])

    # build the Request
    headers = {
        'Content-type': content_type,
        'Content-length': str(len(body))
    }
    req = Request(config.repository, body, headers)

    # handle HTTP and include the Basic Auth handler
    opener = build_opener(HTTPBasicAuthHandler(password_mgr=auth))
    try:
        opener.open(req)
    except HTTPError:
        e = extract_exception()
        code, msg = e.code, e.msg
    except URLError:
        e = extract_exception()
        code, msg = 500, str(e)
    else:
        code, msg = 200, 'OK'

    return code, msg



import os

from copy \
    import \
        deepcopy

from bento.core.pkg_objects \
    import \
        Extension, DataFiles, Executable, CompiledLibrary
from bento.core.meta \
    import \
        _set_metadata, _METADATA_FIELDS
from bento.parser.misc \
    import \
        build_ast_from_raw_dict, raw_parse
from bento.compat.api \
    import \
        relpath
from bento.core.subpackage \
    import \
        SubPackageDescription
from bento.core.parse_helpers \
    import \
        extract_top_dicts, extract_top_dicts_subento
from bento.errors \
    import \
        InvalidPackage, InternalBentoError
import bento.utils.path

def _parse_libraries(libraries):
    ret = {}
    if len(libraries) > 0:
        if not list(libraries.keys()) == ["default"]:
            raise NotImplementedError(
                    "Non default library not yet supported")

        default = libraries["default"]
        for k in ["packages", "py_modules", "install_requires", "sub_directory"]:
            if k in default:
                ret[k] = default[k]

        ret["extensions"] = {}
        for k, v in default.get("extensions", {}).items():
            ret["extensions"][k] = Extension.from_parse_dict(v)

        ret["compiled_libraries"] = {}
        for k, v in default.get("compiled_libraries", {}).items():
            ret["compiled_libraries"][k] = \
                    CompiledLibrary.from_parse_dict(v)
    return ret

def recurse_subentos(subentos, source_dir):
    filenames = []
    subpackages = {}

    # FIXME: this is damn ugly - using nodes would be good here
    def _recurse(subento, cwd):
        f = os.path.normpath(os.path.join(cwd, subento, "bento.info"))
        if not os.path.exists(f):
            raise ValueError("%s not found !" % f)
        filenames.append(relpath(f, source_dir))

        fid = open(f)
        try:
            key = relpath(f, source_dir)
            rdir = relpath(os.path.join(cwd, subento), source_dir)

            d = raw_parse(fid.read(), f)
            kw, subentos = raw_to_subpkg_kw(d)
            subpackages[key] = SubPackageDescription(rdir, **kw)
            hooks_as_abspaths = [os.path.normpath(os.path.join(cwd, subento, h)) \
                                 for h in subpackages[key].hook_files]
            filenames.extend([relpath(f, source_dir) for f in hooks_as_abspaths])
            for s in subentos:
                _recurse(s, os.path.join(cwd, subento))
        finally:
            fid.close()

    for s in subentos:
        _recurse(s, source_dir)
    return subpackages, filenames

def build_libs_from_dict(libraries_d):
    return _parse_libraries(libraries_d)

def build_executables_from_dict(executables_d):
    executables = {}
    for name, executable in executables_d.items():
        executables[name] = Executable.from_parse_dict(executable)
    return executables

def build_data_files_from_dict(data_files_d):
    data_files = {}
    for name, data_file_d in data_files_d.items():
        data_files[name] = DataFiles.from_parse_dict(data_file_d)
    return data_files

def raw_to_subpkg_kw(raw_dict):
    d = build_ast_from_raw_dict(raw_dict)

    libraries_d, misc_d = extract_top_dicts_subento(deepcopy(d))

    kw = {}
    libraries = build_libs_from_dict(libraries_d)
    kw.update(libraries)
    kw["hook_files"] = misc_d["hook_files"]
    sub_directory = kw.pop("sub_directory")
    if sub_directory is not None:
        raise InternalBentoError("Unexpected sub_directory while parsing recursed bendo")

    return kw, misc_d["subento"]

def raw_to_pkg_kw(raw_dict, user_flags, bento_info=None):
    if bento_info is None:
        source_dir = os.getcwd()
    else:
        # bento_info may be a string or a node
        try:
            source_dir = bento_info.parent.abspath()
            bento_info_path = bento_info.srcpath()
        except AttributeError:
            if os.path.isabs(bento_info):
                source_dir = os.path.dirname(bento_info)
                bento_info_path = os.path.basename(bento_info)
            else:
                source_dir = os.getcwd()
                bento_info_path = os.path.basename(bento_info)
                assert bento_info_path == bento_info

    d = build_ast_from_raw_dict(raw_dict, user_flags)

    meta_d, libraries_d, options_d, misc_d = extract_top_dicts(deepcopy(d))
    libraries = build_libs_from_dict(libraries_d)
    executables = build_executables_from_dict(misc_d.pop("executables"))
    data_files = build_data_files_from_dict(misc_d.pop("data_files"))

    kw = {}
    kw.update(meta_d)
    for k in libraries:
        kw[k] = libraries[k]
    kw["executables"] = executables
    kw["data_files"] = data_files

    misc_d.pop("path_options")
    misc_d.pop("flag_options")

    if "subento" in misc_d:
        subentos = misc_d.pop("subento")
        if len(subentos) > 0 and libraries and libraries["sub_directory"] is not None:
            raise InvalidPackage("You cannot use both Recurse and Library:SubDirectory features !")
        else:
            subpackages, files = recurse_subentos(subentos, source_dir=source_dir)
            kw["subpackages"] = subpackages
    else:
        files = []

    kw.update(misc_d)
    if bento_info is not None:
        files.append(bento_info_path)
    files.extend(misc_d["hook_files"])
    # XXX: Do we want to automatically add the hook and bento files in extra
    # source files at the PackageDescription level ?
    kw["extra_source_files"].extend(files)

    if "description_from_file" in kw:
        if bento_info:
            description_file = os.path.join(source_dir, kw["description_from_file"])
        else:
            description_file = kw["description_from_file"]
        if not os.path.exists(description_file):
            raise IOError("Description file %r not found" % (description_file,))
        else:
            f = open(description_file)
            try:
                kw["description"] = f.read()
            finally:
                f.close()
    return kw, files

class PackageDescription:
    @classmethod
    def __from_data(cls, data, user_flags, filename=None):
        if not user_flags:
            user_flags = {}

        d = raw_parse(data, filename)
        kw, files = raw_to_pkg_kw(d, user_flags, filename)
        return cls(**kw)

    @classmethod
    def from_string(cls, s, user_flags=None):
        """Create a PackageDescription from a string containing the package
        description."""
        return cls.__from_data(s, user_flags)

    @classmethod
    def from_file(cls, filename, user_flags=None):
        """Create a PackageDescription from a bento.info file."""
        info_file = open(filename, 'r')
        try:
            data = info_file.read()
            ret = cls.__from_data(data, user_flags, filename)
            return ret
        finally:
            info_file.close()

    # FIXME: this stuff has passed the uglyness threshold quite some time
    # ago...
    def __init__(self, name=None, version=None, summary=None, url=None,
            author=None, author_email=None, maintainer=None,
            maintainer_email=None, license=None, description=None,
            platforms=None, packages=None, py_modules=None, extensions=None,
            install_requires=None, build_requires=None,
            download_url=None, extra_source_files=None, data_files=None,
            classifiers=None, provides=None, obsoletes=None, executables=None,
            hook_files=None, config_py=None, compiled_libraries=None,
            subpackages=None, description_from_file=None, meta_template_files=None,
            keywords=None, sub_directory=None, use_backends=None):
        # XXX: should we check that we have sequences when required
        # (py_modules, etc...) ?

        # Package content
        if not packages:
            self.packages = []
        else:
            self.packages = packages

        # Package content
        if not subpackages:
            self.subpackages = {}
        else:
            self.subpackages = subpackages

        if not py_modules:
            self.py_modules = []
        else:
            self.py_modules = py_modules

        if extensions:
            self.extensions = extensions
        else:
            self.extensions = {}
        if compiled_libraries:
            self.compiled_libraries = compiled_libraries
        else:
            self.compiled_libraries = {}

        if not extra_source_files:
            self.extra_source_files = []
        else:
            self.extra_source_files = extra_source_files

        if not data_files:
            self.data_files = {}
        else:
            self.data_files = data_files

        if not executables:
            self.executables = {}
        else:
            self.executables = executables

        self.use_backends = use_backends or []

        pkgs = []
        for p in self.packages:
            pkgs.append(p)
        for p in self.py_modules:
            pkgs.append(p)
        for p in self.extensions.values():
            pkgs.append(p.name)
        top_levels = [i for i in pkgs if not "." in i]

        # Package metadata
        _args = locals()
        kw = dict([(k, _args[k]) for k in _METADATA_FIELDS if k in _args])
        _set_metadata(self, **kw)

        if hook_files is not None:
            self.hook_files = hook_files
        else:
            self.hook_files = []

        if config_py is not None and os.sep != "/":
            self.config_py = bento.utils.path.unnormalize_path(config_py)
        else:
            self.config_py = config_py

        if meta_template_files is None:
            meta_template_files = []
        self.meta_template_files = [bento.utils.path.unnormalize_path(f) for f in meta_template_files]

        self.extra_source_files.extend(self.meta_template_files)

        self.sub_directory = sub_directory

def static_representation(pkg, options={}):
    """Return the static representation of the given PackageDescription
    instance as a string."""
    indent_level = 4
    r = []

    def indented_list(head, seq, ind):
        r.append("%s%s:" % (' ' * (ind - 1) * indent_level, head))
        r.append(',\n'.join([' ' * ind * indent_level + i for i in seq]))

    if pkg.name:
        r.append("Name: %s" % pkg.name)
    if pkg.version:
        r.append("Version: %s" % pkg.version)
    if pkg.summary:
        r.append("Summary: %s" % pkg.summary)
    if pkg.url:
        r.append("Url: %s" % pkg.url)
    if pkg.download_url:
        r.append("DownloadUrl: %s" % pkg.download_url)
    if pkg.description:
        lines = pkg.description.splitlines()
        description = [lines[0]]
        if len(lines) > 1:
            description.extend([' ' * indent_level  + line for line in lines[1:]])
        r.append("Description: %s" % "\n".join(description))
    if pkg.author:
        r.append("Author: %s" % pkg.author)
    if pkg.author_email:
        r.append("AuthorEmail: %s" % pkg.author_email)
    if pkg.maintainer:
        r.append("Maintainer: %s" % pkg.maintainer)
    if pkg.maintainer_email:
        r.append("MaintainerEmail: %s" % pkg.maintainer_email)
    if pkg.license:
        r.append("License: %s" % pkg.license)
    if pkg.platforms:
        r.append("Platforms: %s" % ",".join(pkg.platforms))
    if pkg.classifiers:
        indented_list("Classifiers", pkg.classifiers, 1)

    if options:
        for k in options:
            if k == "path_options":
                for p in options["path_options"]:
                    r.append('')
                    r.append("Path: %s" % p.name)
                    r.append(' ' * indent_level + "Description: %s" % p.description)
                    r.append(' ' * indent_level + "Default: %s" % p.default_value)
            else:
                raise ValueError("Gne ? %s" % k)
        r.append('')

    if pkg.extra_source_files:
        indented_list("ExtraSourceFiles", pkg.extra_source_files, 1)
        r.append('')

    if pkg.data_files:
        for section in pkg.data_files.values():
            r.append("DataFiles: %s" % section.name)
            r.append(' ' * indent_level + "SourceDir: %s" % section.source_dir)
            r.append(' ' * indent_level + "TargetDir: %s" % section.target_dir)
            indented_list("Files", section.files, 2)
            r.append('')

    # Fix indentation handling instead of hardcoding it
    if pkg.py_modules or pkg.packages or pkg.extensions:
        r.append("Library:")

        if pkg.install_requires:
            indented_list("InstallRequires", pkg.install_requires, 2)
        if pkg.py_modules:
            indented_list("Modules", pkg.py_modules, 2)
        if pkg.packages:
            indented_list("Packages", pkg.packages, 2)

        if pkg.extensions:
            for name, ext in pkg.extensions.items():
                r.append(' ' * indent_level + "Extension: %s" % name)
                indented_list("Sources", ext.sources, 3)
                if ext.include_dirs:
                    indented_list("IncludeDirs", ext.include_dirs, 3)
        r.append("")

    for name, value in pkg.executables.items():
        r.append("Executable: %s" % name)
        r.append(' ' * indent_level + "Module: %s" % value.module)
        r.append(' ' * indent_level + "Function: %s" % value.function)
        r.append("")
    return "\n".join(r)



def has_cython_code(pkg):
    for extension in pkg.extensions.values():
        for source in extension.sources:
            if source.endswith("pyx"):
                return True

    if pkg.subpackages:
        for spkg in pkg.subpackages.values():
            for extension in spkg.extensions.values():
                for source in extension.sources:
                    if source.endswith("pyx"):
                        return True
    return False

def has_compiled_code(pkg):
    has_compiled_code = len(pkg.extensions) > 0 or len(pkg.compiled_libraries) > 0
    if not has_compiled_code:
        if pkg.subpackages:
            for v in pkg.subpackages.values():
                if len(v.extensions) > 0 or len(v.compiled_libraries) > 0:
                    has_compiled_code = True
                    break
    return has_compiled_code



from bento.errors \
    import \
        InvalidPackage
from bento.core.meta \
    import \
        _METADATA_FIELDS

def extract_top_dicts(d):
    """Given an "abstract" dictionary returned by parse_to_dict, build
    meta, library, options and misc dictionaries."""
    meta = {}
    misc = {"extra_source_files": [],
            "executables": {},
            "data_files": {},
            "hook_files": [],
            "config_py": None,
            "meta_template_files": [],
            "flag_options": [],
            "path_options": [],
            "subento": [],
            "use_backends": []}
    options = {}

    for k in _METADATA_FIELDS:
        if k in d:
            meta[k] = d.pop(k)
    if "libraries" in d:
        libraries = d.pop("libraries")
    else:
        libraries = {}
    for k in misc.keys():
        if k in d:
            misc[k] = d.pop(k)

    if len(d) > 0:
        raise ValueError("Unknown entry(ies) %s" % d.keys())

    return meta, libraries, options, misc

def extract_top_dicts_subento(d):
    """Given an "abstract" dictionary returned by parse_to_dict, build
    library, and misc dictionaries.
    
    This function should be used for subentos"""
    misc = {"subento": [], "hook_files": []}

    if "libraries" in d:
        libraries = d.pop("libraries")
    else:
        libraries = {}
    # FIXME: bento vs subento visitor. Those should not be defined in the first
    # place for subento.
    for library in libraries.values():
        for k, field_name in [("install_requires", "InstallRequires"), ("py_modules", "Modules")]:
            v = library.pop(k)
            if len(v) > 0:
                raise InvalidPackage("Invalid entry %r in recursed bento file(s)" % field_name)
    for k in misc.keys():
        if k in d:
            misc[k] = d.pop(k)

    # FIXME: bento vs subento visitor. Those should not be defined in the first
    # place for subento.
    for k in ["path_options", "flag_options", "data_files", "extra_source_files", "executables"]:
        v = d.pop(k)
        if len(v) > 0:
            raise ValueError("Invalid non empty entry %s" % k)
    if len(d) > 0:
        raise ValueError("Unknown entry(ies) %s" % d.keys())

    return libraries, misc



import os

from yaku.task_manager \
    import \
        extension, get_extension_hook
from yaku.task \
    import \
        task_factory
from yaku.compiled_fun \
    import \
        compile_fun
from yaku.utils \
    import \
        ensure_dir

swig_func, swig_vars = compile_fun("swig", "${SWIG} ${SWIGFLAGS} -o ${TGT[0]} ${SRC}", False)

@extension(".i")
def swig_hook(self, name):
    # FIXME: only handle C extension (no C++)
    base = os.path.splitext(name)[0]
    target = os.path.join(self.env["BLDDIR"], base)
    targets = [target + "_wrap.c", target + ".py"]
    for t in targets:
        ensure_dir(t)
    task = task_factory("swig")(inputs=name, outputs=targets)
    task.func = swig_func
    task.env_vars = swig_vars
    task.env = self.env

    compile_task = get_extension_hook(".c")
    ctask = compile_task(self, targets[0])
    return [task] + ctask



import sys
import re
import os

from yaku.compat.rename \
    import \
        rename
from yaku.compat.py3k \
    import \
        get_exception

def ensure_dir(path):
    dirname = os.path.dirname(path)
    if dirname and not os.path.exists(dirname):
        os.makedirs(dirname)

re_inc = re.compile(\
    '^[ \t]*(#|%:)[ \t]*(include)[ \t]*(.*)\r*$',
    re.IGNORECASE | re.MULTILINE)


RE_INCLUDE = re.compile('^\s*(<(?P<a>.*)>|"(?P<b>.*)")')

re_nl = re.compile('\\\\\r*\n', re.MULTILINE)
re_cpp = re.compile(\
    r"""(/\*[^*]*\*+([^/*][^*]*\*+)*/)|//[^\n]*|("(\\.|[^"\\])*"|'(\\.|[^'\\])*'|.[^/"'\\]*)""",
    re.MULTILINE)

def repl(m):
    s = m.group(1)
    if s is not None: return ' '
    s = m.group(3)
    if s is None: return ''
    return s

def extract_include(txt, defs):
    """process a line in the form "#include foo" to return a string representing the file"""
    m = RE_INCLUDE.search(txt)
    if m:
        if m.group('a'):
            return '<', m.group('a')
        if m.group('b'):
            return '"', m.group('b')

    return None, None

def lines_includes(filename):
    code = open(filename).read()
    #if use_trigraphs:
    #   for (a, b) in trig_def: code = code.split(a).join(b)
    code = re_nl.sub('', code)
    code = re_cpp.sub(repl, code)
    return [(m.group(2), m.group(3)) for m in re.finditer(re_inc, code)]

def find_deps(node, cpppaths=["/usr/include", "."]):
    nodes = []
    names = []

    def _find_deps(node):
        lst = lines_includes(node)

        for (_, line) in lst:
            t, filename = extract_include(line, None)
            if t is None:
                continue
            if filename in names:
                continue

            found = None
            for n in cpppaths:
                if found:
                    break
                if os.path.exists(os.path.join(n, filename)):
                    found = os.path.join(n, filename)
                #else:
                #    # XXX: most likely wrong
                #    found = os.path.join(n, filename)
                #    nodes.append(found)

            if not found:
                if not filename in names:
                    names.append(filename)
            elif not found in nodes:
                nodes.append(found)
                _find_deps(found)

    _find_deps(node)
    return nodes

def find_program(program, path_list=None):
    if path_list is None:
        path_list = os.environ["PATH"].split(os.pathsep)

    for p in path_list:
        ppath = os.path.join(p, program)
        if sys.platform == "win32":
            for ext in [".exe"]:
                epath = ppath + ext
                if os.path.exists(epath):
                    return epath
        else:
            if os.path.exists(ppath):
                return ppath

    return None

if sys.version_info[0] < 3:
    from yaku._utils_py2 import join_bytes, function_code
    def is_string(s):
        return isinstance(s, basestring)
else:
    from yaku._utils_py3 import join_bytes, function_code
    def is_string(s):
        return isinstance(s, str)

def extract_exception():
    """Extract the last exception.

    Used to avoid the except ExceptionType as e, which cannot be written the
    same across supported versions. I.e::

        try:
            ...
        except Exception, e:
            ...

    becomes:

        try:
            ...
        except Exception:
            e = extract_exception()
    """
    return sys.exc_info()[1]



import os
import sys
import copy

from yaku._config \
    import \
        TOOLDIRS
from yaku.task_manager \
    import \
        CompiledTaskGen, TaskGen
from yaku.errors \
    import \
        TaskRunFailure
from yaku.scheduler \
    import \
        run_tasks
from yaku.conf \
    import \
        with_conf_blddir, create_file, write_log
from yaku.utils \
    import \
        get_exception

def import_tools(tool_list, tooldirs=None):
    old_sys = sys.path[:]
    if tooldirs is not None:
        sys.path = tooldirs + TOOLDIRS + sys.path
    else:
        sys.path = TOOLDIRS + sys.path

    try:
        ret = {}
        for t in tool_list:
            ret[t] = __import__(t)
        return ret
    finally:
        sys.path = old_sys

class Builder(object):
    def __init__(self, ctx):
        self.configured = False
        self.ctx = ctx
        self.env = copy.deepcopy(ctx.env)

    def to_nodes(self, filenames):
        """Convert source filenames to nodes.

        Parameters
        ----------
        :filenames: str|sequence, list of filenames. Automatically converted to a list
            if filenames is a string"""
        if isinstance(filenames, str):
            filenames = [filenames]
        nodes = []
        for s in filenames:
            n = self.ctx.path.find_resource(s)
            if n is None:
                raise IOError("Source file %s not found" % s)
            else:
                nodes.append(n)
        return nodes

    def configure(self):
        pass

    def _task_gen_factory(self, name, target, sources, env):
        sources = self.to_nodes(sources)

        task_gen = TaskGen(name, self.ctx, sources, target)
        task_gen.env = _merge_env(self.env, env)
        tasks = task_gen.process()
        for t in tasks:
            t.env = task_gen.env
        self.ctx.tasks.extend(tasks)

        outputs = tasks[0].outputs[:]
        return outputs

def try_task_maker(conf, task_maker, name, body, headers, env=None):
    if headers:
        head = "\n".join(["#include <%s>" % h for h in headers])
    else:
        head = ""
    code = "\n".join([c for c in [head, body]])
    sources = [create_file(conf, code, name, ".c")]

    task_gen = CompiledTaskGen("conf", conf, sources, name)
    task_gen.env.update(copy.deepcopy(conf.env))
    task_gen.env = _merge_env(task_gen.env, env)
    task_gen.env.prepend("LIBDIR", conf.path.declare(".").abspath())

    tasks = task_maker(task_gen, name)
    conf.last_task = tasks[-1]

    for t in tasks:
        t.disable_output = True
        t.log = conf.log

    succeed = False
    explanation = None
    try:
        try:
            run_tasks(conf, tasks)
            succeed = True
        except TaskRunFailure:
            e = get_exception()
            explanation = str(e)
            #raise
    finally:
        write_log(conf, conf.log, tasks, code, succeed, explanation)
    return succeed

def _merge_env(_env, new_env):
    if new_env is not None:
        ret = copy.copy(_env)
        for k, v in new_env.items():
            if k in ret and hasattr(ret[k], "extend"):
                old = copy.copy(_env[k])
                old.extend(v)
                ret[k] = old
            else:
                ret[k] = v
        return ret
    else:
        return _env




import os

import os.path as op

from bento.utils.utils \
    import \
        subst_vars
from bento.installed_package_description \
    import \
        InstalledPkgDescription, ipkg_meta_from_pkg
from bento._config \
    import \
        IPKG_PATH

from bento.commands.core \
    import \
        Option
from bento.commands.core \
    import \
        Command

class SectionWriter(object):
    def __init__(self):
        self.sections = {}

    def store(self, filename, pkg):
        meta = ipkg_meta_from_pkg(pkg)
        p = InstalledPkgDescription(self.sections, meta, pkg.executables)
        if not op.exists(op.dirname(filename)):
            os.makedirs(op.dirname(filename))
        p.write(filename)

class BuildCommand(Command):
    long_descr = """\
Purpose: build the project
Usage:   bentomaker build [OPTIONS]."""
    short_descr = "build the project."
    common_options = Command.common_options \
                        + [Option("-i", "--inplace",
                                  help="Build extensions in place", action="store_true"),
                           Option("-j", "--jobs",
                                  help="Parallel builds (yaku build only - EXPERIMENTAL)",
                                  dest="jobs"),
                           Option("-v", "--verbose",
                                  help="Verbose output (yaku build only)",
                                  action="store_true")]

    def run(self, ctx):
        p = ctx.options_context.parser
        o, a = p.parse_args(ctx.command_argv)
        if o.help:
            p.print_help()
            return

        ctx.compile()
        ctx.post_compile()

    def finish(self, ctx):
        super(BuildCommand, self).finish(ctx)
        n = ctx.build_node.make_node(IPKG_PATH)
        ctx.section_writer.store(n.abspath(), ctx.pkg)

def _config_content(paths):
    keys = sorted(paths.keys())
    n = max([len(k) for k in keys]) + 2
    content = []
    for name, value in sorted(paths.items()):
        content.append('%s = %r' % (name.upper().ljust(n), subst_vars(value, paths)))
    return "\n".join(content)




import os
import os.path as op
import tempfile
import shutil
import zipfile

from bento.compat.api.moves \
    import \
        unittest
from bento.core.package \
    import \
        PackageDescription
from bento.core.node \
    import \
        create_root_with_source_tree
from bento.commands.options \
    import \
        OptionsContext
from bento.commands.command_contexts \
    import \
        SdistContext
from bento.commands.sdist \
    import \
        SdistCommand
from bento.commands.wrapper_utils \
    import \
        run_command_in_context
from bento.core.testing \
    import \
        create_fake_package_from_bento_infos, create_fake_package_from_bento_info
from bento.convert.utils \
    import \
        canonalize_path

class TestBaseSdist(unittest.TestCase):
    def setUp(self):
        self.save = os.getcwd()
        self.d = tempfile.mkdtemp()
        os.chdir(self.d)
        try:
            self.root = create_root_with_source_tree(self.d, os.path.join(self.d, "build"))
            self.top_node = self.root._ctx.srcnode
            self.build_node = self.root._ctx.bldnode
            self.run_node = self.root.find_node(self.d)
        except Exception:
            os.chdir(self.save)
            raise

    def tearDown(self):
        os.chdir(self.save)
        shutil.rmtree(self.d)

    def _assert_archive_equality(self, archive, r_archive_list):
        r_archive_list = set(canonalize_path(f) for f in r_archive_list)
        archive = self.run_node.find_node(archive)
        z = zipfile.ZipFile(archive.abspath(), "r")
        try:
            archive_list = set(z.namelist())
            self.assertEqual(archive_list, r_archive_list)
        finally:
            z.close()

    def test_simple_package(self):
        bento_info = """\
Name: foo
Version: 1.0

ExtraSourceFiles: yeah.info

Library:
    Packages: foo, foo.bar
    Modules: fubar
"""
        archive_list = [op.join("foo-1.0", f) for f in ["yeah.info",
                                                        op.join("foo", "__init__.py"),
                                                        op.join("foo", "bar", "__init__.py"),
                                                        "fubar.py"]]

        create_fake_package_from_bento_info(self.top_node, bento_info)
        package = PackageDescription.from_string(bento_info)

        sdist = SdistCommand()
        opts = OptionsContext.from_command(sdist)
        cmd_argv = ["--output-file=foo.zip", "--format=zip"]

        context = SdistContext(None, cmd_argv, opts, package, self.run_node)
        run_command_in_context(context, sdist)

        self._assert_archive_equality(op.join("dist", "foo.zip"), archive_list)

    def test_extra_source_registration(self):
        bento_info = """\
Name: foo
Version: 1.0

Library:
    Modules: fubar
"""
        archive_list = [op.join("foo-1.0", f) for f in ["fubar.py", "yeah.info"]]

        extra_node = self.top_node.make_node("yeah.info")
        extra_node.write("")

        create_fake_package_from_bento_info(self.top_node, bento_info)
        package = PackageDescription.from_string(bento_info)

        sdist = SdistCommand()
        opts = OptionsContext.from_command(sdist)
        cmd_argv = ["--output-file=foo.zip", "--format=zip"]

        context = SdistContext(None, cmd_argv, opts, package, self.run_node)
        context.register_source_node(self.top_node.find_node("yeah.info"))
        run_command_in_context(context, sdist)

        self._assert_archive_equality(op.join("dist", "foo.zip"), archive_list)

    def test_extra_source_with_alias(self):
        bento_info = """\
Name: foo
Version: 1.0

Library:
    Modules: fubar
"""
        archive_list = [op.join("foo-1.0", f) for f in ["fubar.py", "bohou.info"]]

        extra_node = self.top_node.make_node("yeah.info")
        extra_node.write("")

        create_fake_package_from_bento_info(self.top_node, bento_info)
        package = PackageDescription.from_string(bento_info)

        sdist = SdistCommand()
        opts = OptionsContext.from_command(sdist)
        cmd_argv = ["--output-file=foo.zip", "--format=zip"]

        context = SdistContext(None, cmd_argv, opts, package, self.run_node)
        context.register_source_node(self.top_node.find_node("yeah.info"), archive_name="bohou.info")
        run_command_in_context(context, sdist)

        self._assert_archive_equality(op.join("dist", "foo.zip"), archive_list)

    def test_template_filling(self):
        bento_info = """\
Name: foo
Version: 1.0

MetaTemplateFile: release.py.in

Library:
    Modules: fubar
"""
        archive_list = [op.join("foo-1.0", f) for f in ["fubar.py", "release.py.in", "release.py"]]

        template = self.top_node.make_node("release.py.in")
        template.write("""\
NAME = $NAME
VERSION = $VERSION
""")

        create_fake_package_from_bento_info(self.top_node, bento_info)
        package = PackageDescription.from_string(bento_info)

        sdist = SdistCommand()
        opts = OptionsContext.from_command(sdist)
        cmd_argv = ["--output-file=foo.zip", "--format=zip"]

        context = SdistContext(None, cmd_argv, opts, package, self.run_node)
        run_command_in_context(context, sdist)

        self._assert_archive_equality(op.join("dist", "foo.zip"), archive_list)



from yaku.tests.test_helpers \
    import \
        TmpContextBase
from yaku.context \
    import \
        get_cfg, get_bld
from yaku.conftests \
    import \
        check_compiler

class ContextTest(TmpContextBase):
    def test_load_store_simple(self):
        """Test one can load and store the contexts."""
        ctx = get_cfg()
        ctx.store()

        ctx = get_bld()
        ctx.store()

class SimpleCCTest(TmpContextBase):
    def test_compiler(self):
        ctx = get_cfg()
        ctx.store()

        ctx = get_bld()
        ctx.store()




import os
import sys

from six.moves import cPickle

from bento.compat.api \
    import \
        defaultdict
import bento.utils.io2

def _invert_dependencies(deps):
    """Given a dictionary of edge -> dependencies representing a DAG, "invert"
    all the dependencies."""
    ideps = {}
    for k, v in deps.items():
        for d in v:
            l = ideps.get(d, None)
            if l:
                l.append(k)
            else:
                l = [k]
            ideps[d] = l

    return ideps

class CommandScheduler(object):
    def __init__(self):
        self.before = defaultdict(list)
        self.command_names = {}

    def _register(self, command_name):
        if not command_name in self.command_names:
            self.command_names[command_name] = command_name

    def set_before(self, command_name, command_name_prev):
        """Set command_name_prev to be run before command_name"""
        self._register(command_name)
        self._register(command_name_prev)
        if not command_name_prev in self.before[command_name]:
            self.before[command_name].append(command_name_prev)

    def set_after(self, command_name, command_name_next):
        """Set command_name_next to be run after command_name"""
        return self.set_before(command_name_next, command_name)

    def order(self, target):
        after = _invert_dependencies(self.before)

        visited = {}
        out = []

        # DFS-based topological sort: this is better to only get the
        # dependencies of a given target command instead of sorting the whole
        # dag
        def _visit(n, stack_visited):
            if n in stack_visited:
                raise ValueError("Cycle detected: %r" % after)
            else:
                stack_visited[n] = None
            if not n in visited:
                visited[n] = None
                for m, v in after.items():
                    if n in v:
                        _visit(m, stack_visited)
                out.append(n)
            stack_visited.pop(n)
        _visit(target, {})
        return [self.command_names[o] for o in out[:-1]]

class CommandDataProvider(object):
    @classmethod
    def from_file(cls, filename):
        """Create a new instance from a pickled file.

        If the file does not exist, creates an instance with no data.
        """
        if os.path.exists(filename):
            fid = open(filename, "rb")
            try:
                return cls(cPickle.load(fid))
            finally:
                fid.close()
        else:
            return cls()

    def store(self, filename):
        bento.utils.io2.safe_write(filename, lambda fid: cPickle.dump(self._data, fid))

    def __init__(self, data=None):
        self._data = defaultdict(list)
        if data:
            self._data.update(data)

    def __getitem__(self, command_name):
        return self._data[command_name]

    def __setitem__(self, command_name, command_argv):
        self._data[command_name] = command_argv



import bento.utils.path
class FlagOption(object):
    def __init__(self, name, default_value, description=None):
        self.name = name
        self.default_value = default_value
        self.description = description

    def __str__(self):
        r = """\
Flag %s
    default value: %s
    description: %s"""
        return r % (self.name, self.default_value, self.description)

class PathOption(object):
    def __init__(self, name, default_value, description=None):
        self.name = name
        self.default_value = default_value
        self.description = description

    def __str__(self):
        r = """\
Customizable path: %s
    default value: %s
    description: %s"""
        return r % (self.name, self.default_value, self.description)

class DataFiles(object):
    @classmethod
    def from_parse_dict(cls, d):
        return cls(**d)

    def __init__(self, name, files=None, target_dir=None, source_dir=None):
        self.name = name

        if files is not None:
            self.files = files
        else:
            self.files = []

        if target_dir is not None:
            self.target_dir = target_dir
        else:
            self.target_dir = "$sitedir"

        if source_dir is not None:
            self.source_dir = source_dir
        else:
            self.source_dir = "."

    def __repr__(self):
        return "DataSection(files=%r, target_dir=%r, source_dir=%r)" % \
               (self.files, self.target_dir, self.source_dir)

    def __eq__(self, other):
        return self.__dict__ == other.__dict__

class Executable(object):
    @classmethod
    def from_parse_dict(cls, d):
        return cls(**d)

    @classmethod
    def from_representation(cls, s):
        if not "=" in s:
            raise ValueError("s should be of the form name=module:function")
        name, value = [j.strip() for j in s.split("=")]
        if not ":" in value:
            raise ValueError(
                "string representation should be of the form module:function, not %s"
                % value)
        module, function = value.split(":", 1)
        return cls(name, module, function)

    def __init__(self, name, module, function):
        # FIXME: check that module is a module name ?
        self.name = name
        self.module = module
        self.function = function

    # FIXME: this function should not really be here...
    def representation(self):
        return ":".join([self.module, self.function])

    # FIXME: this function should not really be here...
    def full_representation(self):
        return "%s = %s" % (self.name, self.representation())

    def __repr__(self):
        return repr({"name": self.name, "module": self.module, "function": self.function})

class Compiled(object):
    @classmethod
    def from_parse_dict(cls, d):
        return cls(**d)

    def __init__(self, name, sources, include_dirs=None):
        self.name = name
        self.sources = [bento.utils.path.normalize_path(p) for p in sources]
        if include_dirs is None:
            self.include_dirs = []
        else:
            self.include_dirs = include_dirs

    def __str__(self):
        return "%s(%s, %s, %s)" % (self.__class__.__name__, self.name,
                                   self.sources, self.include_dirs)

    def __eq__(self, other):
        return self.name == other.name \
                and self.sources == other.sources \
                and self.include_dirs == other.include_dirs

    #def __repr__(self):
    #    return self.__str__()

class Extension(Compiled):
    pass

class CompiledLibrary(Compiled):
    pass



import os
import tempfile
import stat

import os.path as op

from bento.compat.api.moves \
    import \
        unittest
from bento.core.pkg_objects \
    import \
        PathOption, FlagOption, DataFiles
from bento.core.options \
    import \
        PackageOptions
from bento.core.package \
    import \
        PackageDescription
from bento.utils.utils \
    import \
        extract_exception
from bento.compat.api \
    import \
        NamedTemporaryFile
from bento.errors \
    import \
        BentoError, ParseError

from bento.parser import parser as parser_module

#old = sys.path[:]
#try:
#    sys.path.insert(0, join(dirname(__file__), "pkgdescr"))
#    from simple_package import PKG, DESCR
#finally:
#    sys.path = old

class TestParseError(unittest.TestCase):
    def setUp(self):
        self.f = NamedTemporaryFile(mode="w", delete=False)

    def tearDown(self):
        self.f.close()
        os.remove(self.f.name)

    def test_simple(self):
        text = """\
NName: foo
"""
        error_msg = """\
yacc: Syntax error at line 1, Token\(WORD, 'NName'\)
\t'NName: foo'"""
        self.assertRaisesRegexp(ParseError, error_msg, lambda : PackageDescription.from_string(text))

    def test_simple_filename(self):
        f = self.f
        f.write("NName: foo")
        f.flush()
        self.assertRaises(ParseError, lambda : PackageDescription.from_file(f.name))

    def test_error_string(self):
        f = self.f
        f.write("NName: foo")
        f.flush()
        try:
            PackageDescription.from_file(f.name)
            raise AssertionError("Should raise here !")
        except ParseError:
            e = extract_exception()
            self.assertEqual(str(e), """\
  File "%s", line 1
NName: foo
^
Syntax error""" % f.name)

class TestDataFiles(unittest.TestCase):
    def test_simple(self):
        text = """\
Name: foo

DataFiles: data
    TargetDir: $datadir
    Files:
        foo.data
"""
        r_data = DataFiles("data", files=["foo.data"], target_dir="$datadir")
        pkg = PackageDescription.from_string(text)
        self.failUnless("data" in pkg.data_files)
        self.assertEqual(pkg.data_files["data"].__dict__, r_data.__dict__)
    
class TestOptions(unittest.TestCase):
    simple_text = """\
Name: foo

Flag: flag1
    Description: flag1 description
    Default: false

Path: foo
    Description: foo description
    Default: /usr/lib
"""
    def _test_simple(self, opts):
        self.failUnless(opts.name, "foo")

        flag = FlagOption("flag1", "false", "flag1 description")
        self.failUnless(opts.flag_options.keys(), ["flags"])
        self.failUnless(opts.flag_options["flag1"], flag.__dict__)

        path = PathOption("foo", "/usr/lib", "foo description")
        self.failUnless(opts.path_options.keys(), ["foo"])
        self.failUnless(opts.path_options["foo"], path.__dict__)

    def test_simple_from_string(self):
        s = self.simple_text
        opts = PackageOptions.from_string(s)
        self._test_simple(opts)

    def test_simple_from_file(self):
        fid, filename = tempfile.mkstemp(suffix=".info", text=True)
        try:
            os.write(fid, self.simple_text.encode())
            opts = PackageOptions.from_file(filename)
            self._test_simple(opts)
        finally:
            os.close(fid)
            os.remove(filename)

class TestParserCaching(unittest.TestCase):
    def setUp(self):
        wdir = tempfile.mkdtemp()
        self.subwdir= "bar"
        self.old = os.getcwd()
        os.chdir(wdir)

    def tearDown(self):
        os.chdir(self.old)

    def test_no_cached_failure(self):
        """Ensure we raise an error when the cached parser file does not exists
        and we cannot create one."""
        os.makedirs(self.subwdir)
        os.chmod(self.subwdir, stat.S_IREAD | stat.S_IEXEC)
        parsetab = op.join(self.subwdir, "parsetab")

        old_parsetab = parser_module._PICKLED_PARSETAB
        try:
            parser_module._PICKLED_PARSETAB = parsetab
            try:
                parser_module.Parser()
                self.assertTrue(len(self._list_files()) == 0, "Ply created a cached file in CWD")
                self.fail("Expected an error when creating a parser in read-only dir !")
            except BentoError:
                pass
        finally:
            parser_module._PICKLED_PARSETAB = old_parsetab

    def _list_files(self):
        """Return the list of files in cwd (including subdirectories)."""
        created_files = []
        for root, dirs, files in os.walk(os.getcwd()):
            created_files.extend([op.join(root, f) for f in files])
        return created_files

    def test_read_only_cached(self):
        """Test that we can create a parser backed by a read-only parsetab
        file."""
        os.makedirs(self.subwdir)
        parsetab = op.join(self.subwdir, "parsetab")

        old_parsetab = parser_module._PICKLED_PARSETAB
        try:
            parser_module._PICKLED_PARSETAB = parsetab
            parser_module.Parser()
            self.assertEqual(self._list_files(), [op.abspath(parsetab)])

            os.chmod(self.subwdir, stat.S_IREAD | stat.S_IEXEC)
            os.chmod(parsetab, stat.S_IREAD)

            parser_module.Parser()
            # This ensures ply did not write another cached file behind our back
            self.assertEqual(self._list_files(), [op.abspath(parsetab)],
                             "Ply created another cached parsetab file !")
        finally:
            parser_module._PICKLED_PARSETAB = old_parsetab



import os

from bento.compat.api.moves \
    import \
        unittest
from bento.core \
    import \
        PackageDescription
from bento.core.node \
    import \
        create_first_node
from bento.commands.core \
    import \
        HelpCommand, Command
from bento.commands.command_contexts \
    import \
        HelpContext
from bento.commands.wrapper_utils \
    import \
        run_command_in_context
from bento.commands.contexts \
    import \
        GlobalContext
from bento.commands.options \
    import \
        OptionsContext
import bento.commands.registries

class TestHelpCommand(unittest.TestCase):
    def setUp(self):
        self.run_node = create_first_node(os.getcwd())

        registry = bento.commands.registries.CommandRegistry()

        # help command assumes those always exist
        registry.register("configure", Command)
        registry.register("build", Command)
        registry.register("install", Command)
        registry.register("sdist", Command)
        registry.register("build_wininst", Command)
        registry.register("build_egg", Command)
        self.registry = registry

        self.options_registry = bento.commands.registries.OptionsRegistry()
        self.options_registry.register("configure", OptionsContext())

    def test_simple(self):
        help = HelpCommand()
        options = OptionsContext()
        for option in HelpCommand.common_options:
            options.add_option(option)
        global_context = GlobalContext(None,
                commands_registry=self.registry,
                options_registry=self.options_registry)
        pkg = PackageDescription()
        context = HelpContext(global_context, [], options, pkg, self.run_node)

        run_command_in_context(context, help)

    def test_command(self):
        help = HelpCommand()
        options = OptionsContext()
        for option in HelpCommand.common_options:
            options.add_option(option)
        pkg = PackageDescription()

        global_context = GlobalContext(None,
                commands_registry=self.registry,
                options_registry=self.options_registry)
        context = HelpContext(global_context, ["configure"], options, pkg, self.run_node)

        run_command_in_context(context, help)



import os.path as op

from bento.backends.core \
    import \
        AbstractBackend
from bento.commands.command_contexts \
    import \
        ConfigureContext, BuildContext

class DistutilsConfigureContext(ConfigureContext):
    pass

class DistutilsBuildContext(BuildContext):
    def __init__(self, global_context, cmd_argv, options_context, pkg, run_node):
        from bento.commands.build_distutils import DistutilsBuilder
        super(DistutilsBuildContext, self).__init__(global_context, cmd_argv, options_context, pkg, run_node)

        o, a = options_context.parser.parse_args(cmd_argv)
        if o.jobs:
            jobs = int(o.jobs)
        else:
            jobs = 1
        if o.verbose:
            verbose = int(o.verbose)
        else:
            verbose = 0
        self.verbose = verbose
        self.jobs = jobs

        build_path = self.build_node.path_from(self.run_node)
        self._distutils_builder = DistutilsBuilder(verbosity=self.verbose, build_base=build_path)

        def _builder_factory(category, builder):
            def _build(extension, **kw):
                outputs = builder(extension)
                nodes = [self.build_node.find_node(o) for o in outputs]
                from_node = self.build_node

                pkg_dir = op.dirname(extension.name.replace('.', op.sep))
                target_dir = op.join('$sitedir', pkg_dir)
                self.outputs_registry.register_outputs(category, extension.name, nodes,
                                                       from_node, target_dir)
            return _build

        self.builder_registry.register_category("extensions", 
            _builder_factory("extensions", self._distutils_builder.build_extension))
        self.builder_registry.register_category("compiled_libraries",
            _builder_factory("compiled_libraries", self._distutils_builder.build_compiled_library))

    def compile(self):
        super(DistutilsBuildContext, self).compile()
        reg = self.builder_registry
        for category in ("extensions", "compiled_libraries"):
            for name, extension in self._node_pkg.iter_category(category):
                builder = reg.builder(category, name)
                extension = extension.extension_from(extension.ref_node)
                builder(extension)

class DistutilsBackend(AbstractBackend):
    def register_command_contexts(self, context):
        context.register_command_context("configure", DistutilsConfigureContext)
        context.register_command_context("build", DistutilsBuildContext)
